# -*- coding: utf-8 -*-
from django.contrib import admin
from django.http import HttpResponseRedirect
from django.utils.translation import ugettext as _
from koalixcrm.subscriptions.models import *


class AdminSubscriptionEvent(admin.TabularInline):
    model = SubscriptionEvent
    extra = 1
    classes = ('collapse-open',)
    fieldsets = (
        ('Basics', {
            'fields': ('eventdate', 'event',)
        }),
    )
    allow_add = True


class InlineSubscription(admin.TabularInline):
    model = Subscription
    extra = 1
    classes = ('collapse-open',)
    readonly_fields = ('contract', 'subscriptiontype')
    fieldsets = (
        (_('Basics'), {
            'fields': ('contract', 'subscriptiontype')
        }),
    )
    allow_add = False


class OptionSubscription(admin.ModelAdmin):
    list_display = ('id', 'contract', 'subscriptiontype',)
    ordering = ('id', 'contract', 'subscriptiontype')
    search_fields = ('id', 'contract',)
    fieldsets = (
        (_('Basics'), {
            'fields': ('contract', 'subscriptiontype',)
        }),
    )
    inlines = [AdminSubscriptionEvent]

    def createInvoice(self, request, queryset):
        for obj in queryset:
            invoice = obj.createInvoice()
            response = HttpResponseRedirect('/admin/crm/invoice/' + str(invoice.id))
        return response

    def createQuote(self, request, queryset):
        for obj in queryset:
            invoice = obj.createInvoice()
            response = HttpResponseRedirect('/admin/crm/invoice/' + str(invoice.id))
        return response

    def save_model(self, request, obj, form, change):
        if (change == True):
            obj.lastmodifiedby = request.user
        else:
            obj.lastmodifiedby = request.user
            obj.staff = request.user
        obj.save()

    createInvoice.short_description = _("Create Invoice")

    actions = ['createSubscriptionPDF', 'createInvoice']


class OptionSubscriptionType(admin.ModelAdmin):
    list_display = ('id', 'title', 'defaultunit', 'tax', 'accoutingProductCategorie')
    list_display_links = ('id',)
    list_filter = ('title',)
    ordering = ('id', 'title',)
    search_fields = ('id', 'title')
    fieldsets = (
        (_('Basics'), {
            'fields': ('productNumber', 'title', 'description', 'defaultunit', 'tax', 'accoutingProductCategorie',
                       'cancelationPeriod', 'automaticContractExtension', 'automaticContractExtensionReminder',
                       'minimumDuration', 'paymentIntervall', 'contractDocument')
        }),
    )


def createSubscription(a, request, queryset):
    for contract in queryset:
        subscription = Subscription()
        subscription.createSubscriptionFromContract(crmmodels.Contract.objects.get(id=contract.id))
        response = HttpResponseRedirect('/admin/subscriptions/' + str(subscription.id))
    return response


createSubscription.short_description = _("Create Subscription")


class KoalixcrmPluginInterface(object):
    contractInlines = [InlineSubscription]
    contractActions = [createSubscription]
    invoiceInlines = []
    invoiceActions = []
    quoteInlines = []
    quoteActions = []
    customerInlines = []
    customerActions = []


admin.site.register(Subscription, OptionSubscription)
admin.site.register(SubscriptionType, OptionSubscriptionType)



# -*- coding: utf-8 -*-

from datetime import *
from decimal import Decimal
from subprocess import *

from django.conf import settings
from django.contrib import auth
from django.core import serializers
from django.db import models
from django.utils.translation import ugettext as _
from koalixcrm.crm.const.country import *
from koalixcrm.crm.const.postaladdressprefix import *
from koalixcrm.crm.const.purpose import *
from koalixcrm.crm.const.status import *
from koalixcrm.crm.exceptions import OpenInterestAccountMissing
from koalixcrm.crm.exceptions import TemplateSetMissing
from koalixcrm.crm.exceptions import UserExtensionMissing
from koalixcrm.globalSupportFunctions import xstr
from koalixcrm import accounting
from koalixcrm import djangoUserExtension
from lxml import etree


class Currency(models.Model):
    description = models.CharField(verbose_name=_("Description"), max_length=100)
    shortName = models.CharField(verbose_name=_("Displayed Name After Price In The Position"), max_length=3)
    rounding = models.DecimalField(max_digits=5, decimal_places=2, verbose_name=_("Rounding"), blank=True, null=True)

    def __str__(self):
        return self.shortName

    class Meta:
        app_label = "crm"
        verbose_name = _('Currency')
        verbose_name_plural = _('Currency')


class PostalAddress(models.Model):
    prefix = models.CharField(max_length=1, choices=POSTALADDRESSPREFIX, verbose_name=_("Prefix"), blank=True,
                              null=True)
    name = models.CharField(max_length=100, verbose_name=_("Name"), blank=True, null=True)
    prename = models.CharField(max_length=100, verbose_name=_("Prename"), blank=True, null=True)
    addressline1 = models.CharField(max_length=200, verbose_name=_("Addressline 1"), blank=True, null=True)
    addressline2 = models.CharField(max_length=200, verbose_name=_("Addressline 2"), blank=True, null=True)
    addressline3 = models.CharField(max_length=200, verbose_name=_("Addressline 3"), blank=True, null=True)
    addressline4 = models.CharField(max_length=200, verbose_name=_("Addressline 4"), blank=True, null=True)
    zipcode = models.IntegerField(verbose_name=_("Zipcode"), blank=True, null=True)
    town = models.CharField(max_length=100, verbose_name=_("City"), blank=True, null=True)
    state = models.CharField(max_length=100, verbose_name=_("State"), blank=True, null=True)
    country = models.CharField(max_length=2, choices=[(x[0], x[3]) for x in COUNTRIES], verbose_name=_("Country"),
                               blank=True, null=True)

    class Meta:
        app_label = "crm"
        verbose_name = _('Postal Address')
        verbose_name_plural = _('Postal Address')


class PhoneAddress(models.Model):
    phone = models.CharField(max_length=20, verbose_name=_("Phone Number"))

    class Meta:
        app_label = "crm"
        verbose_name = _('Phone Address')
        verbose_name_plural = _('Phone Address')


class EmailAddress(models.Model):
    email = models.EmailField(max_length=200, verbose_name=_("Email Address"))

    class Meta:
        app_label = "crm"
        verbose_name = _('Email Address')
        verbose_name_plural = _('Email Address')


class Contact(models.Model):
    name = models.CharField(max_length=300, verbose_name=_("Name"))
    dateofcreation = models.DateTimeField(verbose_name=_("Created at"), auto_now=True)
    lastmodification = models.DateTimeField(verbose_name=_("Last modified"), auto_now_add=True)
    lastmodifiedby = models.ForeignKey('auth.User', limit_choices_to={'is_staff': True}, blank=True,
                                       verbose_name=_("Last modified by"), editable=True)

    class Meta:
        app_label = "crm"
        verbose_name = _('Contact')
        verbose_name_plural = _('Contact')


class CustomerBillingCycle(models.Model):
    name = models.CharField(max_length=300, verbose_name=_("Name"))
    timeToPaymentDate = models.IntegerField(verbose_name=_("Days To Payment Date"))

    class Meta:
        app_label = "crm"
        verbose_name = _('Customer Billing Cycle')
        verbose_name_plural = _('Customer Billing Cycle')

    def __str__(self):
        return str(self.id) + ' ' + self.name


class CustomerGroup(models.Model):
    name = models.CharField(max_length=300)

    def __str__(self):
        return str(self.id) + ' ' + self.name

    class Meta:
        app_label = "crm"
        verbose_name = _('Customer Group')
        verbose_name_plural = _('Customer Groups')


class Customer(Contact):
    defaultCustomerBillingCycle = models.ForeignKey('CustomerBillingCycle', verbose_name=_('Default Billing Cycle'))
    ismemberof = models.ManyToManyField(CustomerGroup, verbose_name=_('Is member of'), blank=True)

    def createContract(self, request):
        contract = Contract()
        contract.defaultcustomer = self
        contract.defaultcurrency = djangoUserExtension.models.UserExtension.objects.filter(user=request.user.id)[
            0].defaultCurrency
        contract.lastmodifiedby = request.user
        contract.staff = request.user
        contract.save()
        return contract

    def createInvoice(self):
        contract = self.createContract()
        invoice = contract.createInvoice()
        return invoice

    def createQuote(self):
        contract = self.createContract()
        quote = contract.createQuote()
        return quote

    def isInGroup(self, customerGroup):
        for customerGroupMembership in self.ismemberof.all():
            if (customerGroupMembership.id == customerGroup.id):
                return 1
        return 0

    class Meta:
        app_label = "crm"
        verbose_name = _('Customer')
        verbose_name_plural = _('Customers')

    def __str__(self):
        return str(self.id) + ' ' + self.name


class Supplier(Contact):
    offersShipmentToCustomers = models.BooleanField(verbose_name=_("Offers Shipment to Customer"))

    class Meta:
        app_label = "crm"
        verbose_name = _('Supplier')
        verbose_name_plural = _('Supplier')

    def __str__(self):
        return str(self.id) + ' ' + self.name


class Contract(models.Model):
    staff = models.ForeignKey('auth.User', limit_choices_to={'is_staff': True}, blank=True, verbose_name=_("Staff"),
                              related_name="db_relcontractstaff", null=True)
    description = models.TextField(verbose_name=_("Description"))
    defaultcustomer = models.ForeignKey(Customer, verbose_name=_("Default Customer"), null=True, blank=True)
    defaultSupplier = models.ForeignKey(Supplier, verbose_name=_("Default Supplier"), null=True, blank=True)
    defaultcurrency = models.ForeignKey(Currency, verbose_name=_("Default Currency"), blank=False, null=False)
    dateofcreation = models.DateTimeField(verbose_name=_("Created at"), auto_now=True)
    lastmodification = models.DateTimeField(verbose_name=_("Last modified"), auto_now_add=True)
    lastmodifiedby = models.ForeignKey('auth.User', limit_choices_to={'is_staff': True},
                                       verbose_name=_("Last modified by"), related_name="db_contractlstmodified")

    class Meta:
        app_label = "crm"
        verbose_name = _('Contract')
        verbose_name_plural = _('Contracts')

    def createInvoice(self):
        invoice = Invoice()
        invoice.contract = self
        invoice.discount = 0
        invoice.staff = self.staff
        invoice.customer = self.defaultcustomer
        invoice.status = 'C'
        invoice.currency = self.defaultcurrency
        invoice.payableuntil = date.today() + timedelta(
            days=self.defaultcustomer.defaultCustomerBillingCycle.timeToPaymentDate)
        invoice.dateofcreation = date.today().__str__()
        invoice.save()
        return invoice

    def createQuote(self):
        quote = Quote()
        quote.contract = self
        quote.discount = 0
        quote.staff = self.staff
        quote.customer = self.defaultcustomer
        quote.status = 'C'
        quote.currency = self.defaultcurrency
        quote.validuntil = date.today().__str__()
        quote.dateofcreation = date.today().__str__()
        quote.save()
        return quote

    def createPurchaseOrder(self):
        purchaseorder = PurchaseOrder()
        purchaseorder.contract = self
        purchaseorder.description = self.description
        purchaseorder.discount = 0
        purchaseorder.currency = self.defaultcurrency
        purchaseorder.supplier = self.defaultSupplier
        purchaseorder.status = 'C'
        purchaseorder.dateofcreation = date.today().__str__()
        # TODO: today is not correct it has to be replaced
        purchaseorder.save()
        return purchaseorder

    def __str__(self):
        return _("Contract") + " " + str(self.id)


class PurchaseOrder(models.Model):
    contract = models.ForeignKey(Contract, verbose_name=_("Contract"))
    externalReference = models.CharField(verbose_name=_("External Reference"), max_length=100, blank=True, null=True)
    supplier = models.ForeignKey(Supplier, verbose_name=_("Supplier"))
    description = models.CharField(verbose_name=_("Description"), max_length=100, blank=True, null=True)
    lastPricingDate = models.DateField(verbose_name=_("Last Pricing Date"), blank=True, null=True)
    lastCalculatedPrice = models.DecimalField(max_digits=17, decimal_places=2,
                                              verbose_name=_("Last Calculted Price With Tax"), blank=True, null=True)
    lastCalculatedTax = models.DecimalField(max_digits=17, decimal_places=2, verbose_name=_("Last Calculted Tax"),
                                            blank=True, null=True)
    status = models.CharField(max_length=1, choices=PURCHASEORDERSTATUS)
    staff = models.ForeignKey('auth.User', limit_choices_to={'is_staff': True}, blank=True, verbose_name=_("Staff"),
                              related_name="db_relpostaff", null=True)
    currency = models.ForeignKey(Currency, verbose_name=_("Currency"), blank=False, null=False)
    dateofcreation = models.DateTimeField(verbose_name=_("Created at"), auto_now=True)
    lastmodification = models.DateTimeField(verbose_name=_("Last modified"), auto_now_add=True)
    lastmodifiedby = models.ForeignKey('auth.User', limit_choices_to={'is_staff': True},
                                       verbose_name=_("Last modified by"), related_name="db_polstmodified")

    def recalculatePrices(self, pricingDate):
        price = 0
        tax = 0
        try:
            positions = PurchaseOrderPosition.objects.filter(contract=self.id)
            if type(positions) == PurchaseOrderPosition:
                if type(self.discount) == Decimal:
                    price = int(positions.recalculatePrices(pricingDate, self.customer, self.currency) * (
                    1 - self.discount / 100) / self.currency.rounding) * self.currency.rounding
                    tax = int(positions.recalculateTax(self.currency) * (
                    1 - self.discount / 100) / self.currency.rounding) * self.currency.rounding
                else:
                    price = positions.recalculatePrices(pricingDate, self.customer, self.currency)
                    tax = positions.recalculateTax(self.currency)
            else:
                for position in positions:
                    if type(self.discount) == Decimal:
                        price += int(position.recalculatePrices(pricingDate, self.customer, self.currency) * (
                        1 - self.discount / 100) / self.currency.rounding) * self.currency.rounding
                        tax += int(position.recalculateTax(self.currency) * (
                        1 - self.discount / 100) / self.currency.rounding) * self.currency.rounding
                    else:
                        price += position.recalculatePrices(pricingDate, self.customer, self.currency)
                        tax += position.recalculateTax(self.currency)
            self.lastCalculatedPrice = price
            self.lastCalculatedTax = tax
            self.lastPricingDate = pricingDate
            self.save()
            return 1
        except Quote.DoesNotExist as e:
            print("ERROR " + e.__str__())
            print("Der Fehler trat beim File: " + self.sourcefile + " / Cell: " + listOfLines[0][
                                                                                  listOfLines[0].find("cell ") + 4:
                                                                                  listOfLines[0].find(
                                                                                      "(cellType ") - 1] + " auf!")
            exit()
            return 0

    def createPDF(self, whatToExport):
        XMLSerializer = serializers.get_serializer("xml")
        xml_serializer = XMLSerializer()
        out = open(settings.PDF_OUTPUT_ROOT + "purchaseorder_" + str(self.id) + ".xml", "wb")
        objectsToSerialize = list(PurchaseOrder.objects.filter(id=self.id))
        objectsToSerialize += list(Contact.objects.filter(id=self.supplier.id))
        objectsToSerialize += list(Currency.objects.filter(id=self.currency.id))
        objectsToSerialize += list(PurchaseOrderPosition.objects.filter(contract=self.id))
        for position in list(PurchaseOrderPosition.objects.filter(contract=self.id)):
            objectsToSerialize += list(Position.objects.filter(id=position.id))
            objectsToSerialize += list(Product.objects.filter(id=position.product.id))
            objectsToSerialize += list(Unit.objects.filter(id=position.unit.id))
        objectsToSerialize += list(auth.models.User.objects.filter(id=self.staff.id))
        userExtension = djangoUserExtension.models.UserExtension.objects.filter(user=self.staff.id)
        if len(userExtension) == 0:
            raise UserExtensionMissing(_("During PurchaseOrder PDF Export"))
        phoneAddress = djangoUserExtension.models.UserExtensionPhoneAddress.objects.filter(
            userExtension=userExtension[0].id)
        objectsToSerialize += list(userExtension)
        objectsToSerialize += list(phoneAddress)
        templateset = djangoUserExtension.models.TemplateSet.objects.filter(id=userExtension[0].defaultTemplateSet.id)
        if len(templateset) == 0:
            raise TemplateSetMissing(_("During PurchaseOrder PDF Export"))
        objectsToSerialize += list(templateset)
        objectsToSerialize += list(auth.models.User.objects.filter(id=self.lastmodifiedby.id))
        objectsToSerialize += list(PostalAddressForContact.objects.filter(person=self.supplier.id))
        for address in list(PostalAddressForContact.objects.filter(person=self.supplier.id)):
            objectsToSerialize += list(PostalAddress.objects.filter(id=address.id))
        xml_serializer.serialize(objectsToSerialize, stream=out, indent=3)
        out.close()
        check_output(['/usr/bin/fop', '-c', userExtension[0].defaultTemplateSet.fopConfigurationFile.path_full, '-xml',
                      settings.PDF_OUTPUT_ROOT + 'purchaseorder_' + str(self.id) + '.xml', '-xsl',
                      userExtension[0].defaultTemplateSet.purchaseorderXSLFile.xslfile.path_full, '-pdf',
                      settings.PDF_OUTPUT_ROOT + 'purchaseorder_' + str(self.id) + '.pdf'], stderr=STDOUT)
        return settings.PDF_OUTPUT_ROOT + "purchaseorder_" + str(self.id) + ".pdf"

    class Meta:
        app_label = "crm"
        verbose_name = _('Purchase Order')
        verbose_name_plural = _('Purchase Order')

    def __str__(self):
        return _("Purchase Order") + ": " + str(self.id) + " " + _("from Contract") + ": " + str(self.contract.id)


class SalesContract(models.Model):
    contract = models.ForeignKey(Contract, verbose_name=_('Contract'))
    externalReference = models.CharField(verbose_name=_("External Reference"), max_length=100, blank=True)
    discount = models.DecimalField(max_digits=5, decimal_places=2, verbose_name=_("Discount"), blank=True, null=True)
    description = models.CharField(verbose_name=_("Description"), max_length=100, blank=True, null=True)
    lastPricingDate = models.DateField(verbose_name=_("Last Pricing Date"), blank=True, null=True)
    lastCalculatedPrice = models.DecimalField(max_digits=17, decimal_places=2,
                                              verbose_name=_("Last Calculted Price With Tax"), blank=True, null=True)
    lastCalculatedTax = models.DecimalField(max_digits=17, decimal_places=2, verbose_name=_("Last Calculted Tax"),
                                            blank=True, null=True)
    customer = models.ForeignKey(Customer, verbose_name=_("Customer"))
    staff = models.ForeignKey('auth.User', limit_choices_to={'is_staff': True}, blank=True, verbose_name=_("Staff"),
                              related_name="db_relscstaff", null=True)
    currency = models.ForeignKey(Currency, verbose_name=_("Currency"), blank=False, null=False)
    dateofcreation = models.DateTimeField(verbose_name=_("Created at"), auto_now=True)
    lastmodification = models.DateTimeField(verbose_name=_("Last modified"), auto_now_add=True)
    lastmodifiedby = models.ForeignKey('auth.User', limit_choices_to={'is_staff': True},
                                       verbose_name=_("Last modified by"), related_name="db_lstscmodified", null=True,
                                       blank="True")

    def recalculatePrices(self, pricingDate):
        price = 0
        tax = 0
        try:
            positions = SalesContractPosition.objects.filter(contract=self.id)
            if type(positions) == SalesContractPosition:
                if type(self.discount) == Decimal:
                    price = int(positions.recalculatePrices(pricingDate, self.customer, selof.currency) * (
                    1 - self.discount / 100) / self.currency.rounding) * self.currency.rounding
                    tax = int(positions.recalculateTax(self.currency) * (
                    1 - self.discount / 100) / self.currency.rounding) * self.currency.rounding
                else:
                    price = positions.recalculatePrices(pricingDate, self.customer, self.currency)
                    tax = positions.recalculateTax(self.currency)
            else:
                for position in positions:
                    price += position.recalculatePrices(pricingDate, self.customer, self.currency)
                    tax += position.recalculateTax(self.currency)
                if type(self.discount) == Decimal:
                    price = int(price * (1 - self.discount / 100) / self.currency.rounding) * self.currency.rounding
                    tax = int(tax * (1 - self.discount / 100) / self.currency.rounding) * self.currency.rounding

            self.lastCalculatedPrice = price
            self.lastCalculatedTax = tax
            self.lastPricingDate = pricingDate
            self.save()
            return 1
        except Quote.DoesNotExist:
            return 0

    class Meta:
        app_label = "crm"
        verbose_name = _('Sales Contract')
        verbose_name_plural = _('Sales Contracts')

    def __str__(self):
        return _("Sales Contract") + ": " + str(self.id) + " " + _("from Contract") + ": " + str(self.contract.id)


class Quote(SalesContract):
    validuntil = models.DateField(verbose_name=_("Valid until"))
    status = models.CharField(max_length=1, choices=QUOTESTATUS, verbose_name=_('Status'))

    def createInvoice(self):
        invoice = Invoice()
        invoice.contract = self.contract
        invoice.description = self.description
        invoice.discount = self.discount
        invoice.customer = self.customer
        invoice.staff = self.staff
        invoice.status = 'C'
        invoice.derivatedFromQuote = self
        invoice.currency = self.currency
        invoice.payableuntil = date.today() + timedelta(
            days=self.customer.defaultCustomerBillingCycle.timeToPaymentDate)
        invoice.dateofcreation = date.today().__str__()
        invoice.customerBillingCycle = self.customer.defaultCustomerBillingCycle
        invoice.save()
        try:
            quotePositions = SalesContractPosition.objects.filter(contract=self.id)
            for quotePosition in list(quotePositions):
                invoicePosition = SalesContractPosition()
                invoicePosition.product = quotePosition.product
                invoicePosition.positionNumber = quotePosition.positionNumber
                invoicePosition.quantity = quotePosition.quantity
                invoicePosition.description = quotePosition.description
                invoicePosition.discount = quotePosition.discount
                invoicePosition.product = quotePosition.product
                invoicePosition.unit = quotePosition.unit
                invoicePosition.sentOn = quotePosition.sentOn
                invoicePosition.supplier = quotePosition.supplier
                invoicePosition.shipmentID = quotePosition.shipmentID
                invoicePosition.overwriteProductPrice = quotePosition.overwriteProductPrice
                invoicePosition.positionPricePerUnit = quotePosition.positionPricePerUnit
                invoicePosition.lastPricingDate = quotePosition.lastPricingDate
                invoicePosition.lastCalculatedPrice = quotePosition.lastCalculatedPrice
                invoicePosition.lastCalculatedTax = quotePosition.lastCalculatedTax
                invoicePosition.contract = invoice
                invoicePosition.save()
            return invoice
        except Quote.DoesNotExist:
            return

    def createPDF(self, whatToExport):
        XMLSerializer = serializers.get_serializer("xml")
        xml_serializer = XMLSerializer()
        out = open(settings.PDF_OUTPUT_ROOT + "quote_" + str(self.id) + ".xml", "wb")
        objectsToSerialize = list(Quote.objects.filter(id=self.id))
        objectsToSerialize += list(SalesContract.objects.filter(id=self.id))
        objectsToSerialize += list(Contact.objects.filter(id=self.customer.id))
        objectsToSerialize += list(Currency.objects.filter(id=self.currency.id))
        objectsToSerialize += list(SalesContractPosition.objects.filter(contract=self.id))
        for position in list(SalesContractPosition.objects.filter(contract=self.id)):
            objectsToSerialize += list(Position.objects.filter(id=position.id))
            objectsToSerialize += list(Product.objects.filter(id=position.product.id))
            objectsToSerialize += list(Unit.objects.filter(id=position.unit.id))
        objectsToSerialize += list(auth.models.User.objects.filter(id=self.staff.id))
        userExtension = djangoUserExtension.models.UserExtension.objects.filter(user=self.staff.id)
        if (len(userExtension) == 0):
            raise UserExtensionMissing(_("During Quote PDF Export"))
        phoneAddress = djangoUserExtension.models.UserExtensionPhoneAddress.objects.filter(
            userExtension=userExtension[0].id)
        objectsToSerialize += list(userExtension)
        objectsToSerialize += list(PhoneAddress.objects.filter(id=phoneAddress[0].id))
        templateset = djangoUserExtension.models.TemplateSet.objects.filter(id=userExtension[0].defaultTemplateSet.id)
        if (len(templateset) == 0):
            raise TemplateSetMissing(_("During Quote PDF Export"))
        objectsToSerialize += list(templateset)
        objectsToSerialize += list(auth.models.User.objects.filter(id=self.lastmodifiedby.id))
        objectsToSerialize += list(PostalAddressForContact.objects.filter(person=self.customer.id))
        for address in list(PostalAddressForContact.objects.filter(person=self.customer.id)):
            objectsToSerialize += list(PostalAddress.objects.filter(id=address.id))
        xml_serializer.serialize(objectsToSerialize, stream=out, indent=3)
        out.close()
        xml = etree.parse(settings.PDF_OUTPUT_ROOT + "quote_" + str(self.id) + ".xml")
        rootelement = xml.getroot()
        filebrowserdirectory = etree.SubElement(rootelement, "filebrowserdirectory")
        filebrowserdirectory.text = settings.FILEBROWSER_DIRECTORY
        xml.write(settings.PDF_OUTPUT_ROOT + "quote_" + str(self.id) + ".xml")
        if (whatToExport == "quote"):
            check_output(
                ['/usr/bin/fop', '-c', userExtension[0].defaultTemplateSet.fopConfigurationFile.path_full, '-xml',
                 settings.PDF_OUTPUT_ROOT + 'quote_' + str(self.id) + '.xml', '-xsl',
                 userExtension[0].defaultTemplateSet.quoteXSLFile.xslfile.path_full, '-pdf',
                 settings.PDF_OUTPUT_ROOT + 'quote_' + str(self.id) + '.pdf'], stderr=STDOUT)
            return settings.PDF_OUTPUT_ROOT + "quote_" + str(self.id) + ".pdf"
        else:
            check_output(
                ['/usr/bin/fop', '-c', userExtension[0].defaultTemplateSet.fopConfigurationFile.path_full, '-xml',
                 settings.PDF_OUTPUT_ROOT + 'quote_' + str(self.id) + '.xml', '-xsl',
                 userExtension[0].defaultTemplateSet.purchaseconfirmationXSLFile.xslfile.path_full, '-pdf',
                 settings.PDF_OUTPUT_ROOT + 'purchaseconfirmation_' + str(self.id) + '.pdf'], stderr=STDOUT)
            return settings.PDF_OUTPUT_ROOT + "purchaseconfirmation_" + str(self.id) + ".pdf"

    def __str__(self):
        return _("Quote") + ": " + str(self.id) + " " + _("from Contract") + ": " + str(self.contract.id)

    class Meta:
        app_label = "crm"
        verbose_name = _('Quote')
        verbose_name_plural = _('Quotes')


class Invoice(SalesContract):
    payableuntil = models.DateField(verbose_name=_("To pay until"))
    derivatedFromQuote = models.ForeignKey(Quote, blank=True, null=True)
    paymentBankReference = models.CharField(verbose_name=_("Payment Bank Reference"), max_length=100, blank=True,
                                            null=True)
    status = models.CharField(max_length=1, choices=INVOICESTATUS)

    def registerinvoiceinaccounting(self, request):
        dictprices = dict()
        dicttax = dict()
        exists = False
        currentValidAccountingPeriod = accounting.models.AccountingPeriod.getCurrentValidAccountingPeriod()
        activaaccount = accounting.models.Account.objects.filter(isopeninterestaccount=True)
        if len(activaaccount) == 0:
            raise OpenInterestAccountMissing(_("Please specify one open intrest account in the accounting"))
        for position in list(SalesContractPosition.objects.filter(contract=self.id)):
            profitaccount = position.product.accoutingProductCategorie.profitAccount
            dictprices[profitaccount] = position.lastCalculatedPrice
            dicttax[profitaccount] = position.lastCalculatedTax

        for booking in accounting.models.Booking.objects.filter(accountingPeriod=currentValidAccountingPeriod):
            if booking.bookingReference == self:
                raise InvoiceAlreadyRegistered()
        for profitaccount, amount in iter(dictprices.items()):
            booking = accounting.models.Booking()
            booking.toAccount = activaaccount[0]
            booking.fromAccount = profitaccount
            booking.bookingReference = self
            booking.accountingPeriod = currentValidAccountingPeriod
            booking.bookingDate = date.today().__str__()
            booking.staff = request.user
            booking.amount = amount
            booking.lastmodifiedby = request.user
            booking.save()

    def registerpaymentinaccounting(self, request, amount, paymentaccount):
        currentValidAccountingPeriod = accounting.models.AccountingPeriod.getCurrentValidAccountingPeriod()
        activaaccount = accounting.models.Account.objects.filter(isopeninterestaccount=True)
        booking = accounting.models.Booking()
        booking.toAccount = paymentaccount
        booking.fromAccount = activaaccount[0]
        booking.bookingDate = date.today().__str__()
        booking.bookingReference = self
        booking.accountingPeriod = currentValidAccountingPeriod
        booking.amount = self.lastCalculatedPrice
        booking.staff = request.user
        booking.lastmodifiedby = request.user
        booking.save()

    def createPDF(self, whatToExport):
        XMLSerializer = serializers.get_serializer("xml")
        xml_serializer = XMLSerializer()
        out = open(settings.PDF_OUTPUT_ROOT + "invoice_" + str(self.id) + ".xml", "wb")
        objectsToSerialize = list(Invoice.objects.filter(id=self.id))
        objectsToSerialize += list(SalesContract.objects.filter(id=self.id))
        objectsToSerialize += list(Contact.objects.filter(id=self.customer.id))
        objectsToSerialize += list(Currency.objects.filter(id=self.currency.id))
        objectsToSerialize += list(SalesContractPosition.objects.filter(contract=self.id))
        for position in list(SalesContractPosition.objects.filter(contract=self.id)):
            objectsToSerialize += list(Position.objects.filter(id=position.id))
            objectsToSerialize += list(Product.objects.filter(id=position.product.id))
            objectsToSerialize += list(Unit.objects.filter(id=position.unit.id))
        objectsToSerialize += list(auth.models.User.objects.filter(id=self.staff.id))
        userExtension = djangoUserExtension.models.UserExtension.objects.filter(user=self.staff.id)
        if (len(userExtension) == 0):
            raise UserExtensionMissing(_("During Invoice PDF Export"))
        phoneAddress = djangoUserExtension.models.UserExtensionPhoneAddress.objects.filter(
            userExtension=userExtension[0].id)
        objectsToSerialize += list(userExtension)
        objectsToSerialize += list(PhoneAddress.objects.filter(id=phoneAddress[0].id))
        templateset = djangoUserExtension.models.TemplateSet.objects.filter(id=userExtension[0].defaultTemplateSet.id)
        if (len(templateset) == 0):
            raise TemplateSetMissing(_("During Invoice PDF Export"))
        objectsToSerialize += list(templateset)
        objectsToSerialize += list(auth.models.User.objects.filter(id=self.lastmodifiedby.id))
        objectsToSerialize += list(PostalAddressForContact.objects.filter(person=self.customer.id))
        for address in list(PostalAddressForContact.objects.filter(person=self.customer.id)):
            objectsToSerialize += list(PostalAddress.objects.filter(id=address.id))
        xml_serializer.serialize(objectsToSerialize, stream=out, indent=3)
        out.close()
        xml = etree.parse(settings.PDF_OUTPUT_ROOT + "invoice_" + str(self.id) + ".xml")
        rootelement = xml.getroot()
        filebrowserdirectory = etree.SubElement(rootelement, "filebrowserdirectory")
        filebrowserdirectory.text = settings.FILEBROWSER_DIRECTORY
        xml.write(settings.PDF_OUTPUT_ROOT + "invoice_" + str(self.id) + ".xml")
        if (whatToExport == "invoice"):
            check_output(
                ['/usr/bin/fop', '-c', userExtension[0].defaultTemplateSet.fopConfigurationFile.path_full, '-xml',
                 settings.PDF_OUTPUT_ROOT + 'invoice_' + str(self.id) + '.xml', '-xsl',
                 userExtension[0].defaultTemplateSet.invoiceXSLFile.xslfile.path_full, '-pdf',
                 settings.PDF_OUTPUT_ROOT + 'invoice_' + str(self.id) + '.pdf'], stderr=STDOUT)
            return settings.PDF_OUTPUT_ROOT + "invoice_" + str(self.id) + ".pdf"
        else:
            check_output(
                ['/usr/bin/fop', '-c', userExtension[0].defaultTemplateSet.fopConfigurationFile.path_full, '-xml',
                 settings.PDF_OUTPUT_ROOT + 'invoice_' + str(self.id) + '.xml', '-xsl',
                 userExtension[0].defaultTemplateSet.deilveryorderXSLFile.xslfile.path_full, '-pdf',
                 settings.PDF_OUTPUT_ROOT + 'deliveryorder_' + str(self.id) + '.pdf'], stderr=STDOUT)
            return settings.PDF_OUTPUT_ROOT + "deliveryorder_" + str(self.id) + ".pdf"

        #  TODO: def registerPayment(self, amount, registerpaymentinaccounting):

    def __str__(self):
        return _("Invoice") + ": " + str(self.id) + " " + _("from Contract") + ": " + str(self.contract.id)

    class Meta:
        app_label = "crm"
        verbose_name = _('Invoice')
        verbose_name_plural = _('Invoices')


class Unit(models.Model):
    description = models.CharField(verbose_name=_("Description"), max_length=100)
    shortName = models.CharField(verbose_name=_("Displayed Name After Quantity In The Position"), max_length=3)
    isAFractionOf = models.ForeignKey('self', blank=True, null=True, verbose_name=_("Is A Fraction Of"))
    fractionFactorToNextHigherUnit = models.IntegerField(verbose_name=_("Factor Between This And Next Higher Unit"),
                                                         blank=True, null=True)

    def __str__(self):
        return self.shortName

    class Meta:
        app_label = "crm"
        verbose_name = _('Unit')
        verbose_name_plural = _('Units')


class Tax(models.Model):
    taxrate = models.DecimalField(max_digits=5, decimal_places=2, verbose_name=_("Taxrate in Percentage"))
    name = models.CharField(verbose_name=_("Taxname"), max_length=100)
    accountActiva = models.ForeignKey('accounting.Account', verbose_name=_("Activa Account"),
                                      related_name="db_relaccountactiva", null=True, blank=True)
    accountPassiva = models.ForeignKey('accounting.Account', verbose_name=_("Passiva Account"),
                                       related_name="db_relaccountpassiva", null=True, blank=True)

    def getTaxRate(self):
        return self.taxrate;

    def __str__(self):
        return self.name

    class Meta:
        app_label = "crm"
        verbose_name = _('Tax')
        verbose_name_plural = _('Taxes')


class Product(models.Model):
    description = models.TextField(verbose_name=_("Description"), null=True, blank=True)
    title = models.CharField(verbose_name=_("Title"), max_length=200)
    productNumber = models.IntegerField(verbose_name=_("Product Number"))
    defaultunit = models.ForeignKey(Unit, verbose_name=_("Unit"))
    dateofcreation = models.DateTimeField(verbose_name=_("Created at"), auto_now=True)
    lastmodification = models.DateTimeField(verbose_name=_("Last modified"), auto_now_add=True)
    lastmodifiedby = models.ForeignKey('auth.User', limit_choices_to={'is_staff': True},
                                       verbose_name=_("Last modified by"), null=True, blank="True")
    tax = models.ForeignKey(Tax, blank=False)
    accoutingProductCategorie = models.ForeignKey('accounting.ProductCategorie',
                                                  verbose_name=_("Accounting Product Categorie"), null=True,
                                                  blank="True")

    def getPrice(self, date, unit, customer, currency):
        prices = Price.objects.filter(product=self.id)
        unitTransforms = UnitTransform.objects.filter(product=self.id)
        customerGroupTransforms = CustomerGroupTransform.objects.filter(product=self.id)
        validpriceslist = list()
        for price in list(prices):
            for customerGroup in CustomerGroup.objects.filter(customer=customer):
                if price.matchesDateUnitCustomerGroupCurrency(date, unit, customerGroup, currency):
                    validpriceslist.append(price.price);
                else:
                    for customerGroupTransform in customerGroupTransforms:
                        if price.matchesDateUnitCustomerGroupCurrency(date, unit,
                                                                      customerGroupTransfrom.transform(customerGroup),
                                                                      currency):
                            validpriceslist.append(price.price * customerGroup.factor);
                        else:
                            for unitTransfrom in list(unitTransforms):
                                if price.matchesDateUnitCustomerGroupCurrency(date,
                                                                              unitTransfrom.transfrom(unit).transform(
                                                                                      unitTransfrom),
                                                                              customerGroupTransfrom.transform(
                                                                                      customerGroup), currency):
                                    validpriceslist.append(
                                        price.price * customerGroupTransform.factor * unitTransform.factor);
        if (len(validpriceslist) > 0):
            lowestprice = validpriceslist[0]
            for price in validpriceslist:
                if (price < lowestprice):
                    lowestprice = price
            return lowestprice
        else:
            raise Product.NoPriceFound(customer, unit, date, self)

    def getTaxRate(self):
        return self.tax.getTaxRate();

    def __str__(self):
        return str(self.productNumber) + ' ' + self.title

    class Meta:
        app_label = "crm"
        verbose_name = _('Product')
        verbose_name_plural = _('Products')

    class NoPriceFound(Exception):
        def __init__(self, customer, unit, date, product):
            self.customer = customer
            self.unit = unit
            self.date = date
            self.product = product
            return

        def __str__(self):
            return _("There is no Price for this product") + ": " + self.product.__str__() + _(
                "that matches the date") + ": " + self.date.__str__() + " ," + _(
                "customer") + ": " + self.customer.__str__() + _(" and unit") + ":" + self.unit.__str__()


class UnitTransform(models.Model):
    fromUnit = models.ForeignKey('Unit', verbose_name=_("From Unit"), related_name="db_reltransfromfromunit")
    toUnit = models.ForeignKey('Unit', verbose_name=_("To Unit"), related_name="db_reltransfromtounit")
    product = models.ForeignKey('Product', verbose_name=_("Product"))
    factor = models.IntegerField(verbose_name=_("Factor between From and To Unit"), blank=True, null=True)

    def transform(self, unit):
        if (self.fromUnit == unit):
            return self.toUnit
        else:
            return unit

    def __str__(self):
        return "From " + self.fromUnit.shortName + " to " + self.toUnit.shortName

    class Meta:
        app_label = "crm"
        verbose_name = _('Unit Transfrom')
        verbose_name_plural = _('Unit Transfroms')


class CustomerGroupTransform(models.Model):
    fromCustomerGroup = models.ForeignKey('CustomerGroup', verbose_name=_("From Unit"),
                                          related_name="db_reltransfromfromcustomergroup")
    toCustomerGroup = models.ForeignKey('CustomerGroup', verbose_name=_("To Unit"),
                                        related_name="db_reltransfromtocustomergroup")
    product = models.ForeignKey('Product', verbose_name=_("Product"))
    factor = models.IntegerField(verbose_name=_("Factor between From and To Customer Group"), blank=True, null=True)

    def transform(self, customerGroup):
        if (self.fromCustomerGroup == customerGroup):
            return self.toCustomerGroup
        else:
            return unit

    def __str__(self):
        return "From " + self.fromCustomerGroup.name + " to " + self.toCustomerGroup.name

    class Meta:
        app_label = "crm"
        verbose_name = _('Customer Group Price Transfrom')
        verbose_name_plural = _('Customer Group Price Transfroms')


class Price(models.Model):
    product = models.ForeignKey(Product, verbose_name=_("Product"))
    unit = models.ForeignKey(Unit, blank=False, verbose_name=_("Unit"))
    currency = models.ForeignKey(Currency, blank=False, null=False, verbose_name=('Currency'))
    customerGroup = models.ForeignKey(CustomerGroup, blank=True, null=True, verbose_name=_("Customer Group"))
    price = models.DecimalField(max_digits=17, decimal_places=2, verbose_name=_("Price Per Unit"))
    validfrom = models.DateField(verbose_name=_("Valid from"), blank=True, null=True)
    validuntil = models.DateField(verbose_name=_("Valid until"), blank=True, null=True)

    def matchesDateUnitCustomerGroupCurrency(self, date, unit, customerGroup, currency):
        if self.validfrom == None:
            if self.validuntil == None:
                if self.customerGroup == None:
                    if (unit == self.unit) & (self.currency == currency):
                        return 1
                else:
                    if (unit == self.unit) & (self.customerGroup == customerGroup) & (self.currency == currency):
                        return 1
            elif self.customerGroup == None:
                if ((date - self.validuntil).days < 0) & (unit == self.unit) & (self.currency == currency):
                    return 1
            else:
                if ((date - self.validuntil).days < 0) & (unit == self.unit) & (self.customerGroup == customerGroup) & (
                    self.currency == currency):
                    return 1
        elif self.validuntil == None:
            if self.customerGroup == None:
                if ((self.validfrom - date).days < 0) & (unit == self.unit) & (self.currency == currency):
                    return 1
            else:
                if ((self.validfrom - date).days < 0) & (unit == self.unit) & (self.customerGroup == customerGroup) & (
                    self.currency == currency):
                    return 1
        elif self.customerGroup == None:
            if ((self.validfrom - date).days < 0) & (self.validuntil == None) & (unit == self.unit) & (
                self.customerGroup == None) & (self.currency == currency):
                return 1
        else:
            if ((self.validfrom - date).days < 0) & ((date - self.validuntil).days < 0) & (unit == self.unit) & (
                self.customerGroup == customerGroup) & (self.currency == currency):
                return 1

    class Meta:
        app_label = "crm"
        verbose_name = _('Price')
        verbose_name_plural = _('Prices')


class Position(models.Model):
    positionNumber = models.IntegerField(verbose_name=_("Position Number"))
    quantity = models.DecimalField(verbose_name=_("Quantity"), decimal_places=3, max_digits=10)
    description = models.TextField(verbose_name=_("Description"), blank=True, null=True)
    discount = models.DecimalField(max_digits=5, decimal_places=2, verbose_name=_("Discount"), blank=True, null=True)
    product = models.ForeignKey(Product, verbose_name=_("Product"), blank=True, null=True)
    unit = models.ForeignKey(Unit, verbose_name=_("Unit"), blank=True, null=True)
    sentOn = models.DateField(verbose_name=_("Shipment on"), blank=True, null=True)
    supplier = models.ForeignKey(Supplier, verbose_name=_("Shipment Supplier"),
                                 limit_choices_to={'offersShipmentToCustomers': True}, blank=True, null=True)
    shipmentID = models.CharField(max_length=100, verbose_name=_("Shipment ID"), blank=True, null=True)
    overwriteProductPrice = models.BooleanField(verbose_name=_('Overwrite Product Price'))
    positionPricePerUnit = models.DecimalField(verbose_name=_("Price Per Unit"), max_digits=17, decimal_places=2,
                                               blank=True, null=True)
    lastPricingDate = models.DateField(verbose_name=_("Last Pricing Date"), blank=True, null=True)
    lastCalculatedPrice = models.DecimalField(max_digits=17, decimal_places=2, verbose_name=_("Last Calculted Price"),
                                              blank=True, null=True)
    lastCalculatedTax = models.DecimalField(max_digits=17, decimal_places=2, verbose_name=_("Last Calculted Tax"),
                                            blank=True, null=True)

    def recalculatePrices(self, pricingDate, customer, currency):
        if self.overwriteProductPrice == False:
            self.positionPricePerUnit = self.product.getPrice(pricingDate, self.unit, customer, currency)
        if type(self.discount) == Decimal:
            self.lastCalculatedPrice = int(self.positionPricePerUnit * self.quantity * (
            1 - self.discount / 100) / currency.rounding) * currency.rounding
        else:
            self.lastCalculatedPrice = self.positionPricePerUnit * self.quantity
        self.lastPricingDate = pricingDate
        self.save()
        return self.lastCalculatedPrice

    def recalculateTax(self, currency):
        if type(self.discount) == Decimal:
            self.lastCalculatedTax = int(self.product.getTaxRate() / 100 * self.positionPricePerUnit * self.quantity * (
            1 - self.discount / 100) / currency.rounding) * currency.rounding
        else:
            self.lastCalculatedTax = self.product.getTaxRate() / 100 * self.positionPricePerUnit * self.quantity
        self.save()
        return self.lastCalculatedTax

    def __str__(self):
        return _("Position") + ": " + str(self.id)

    class Meta:
        app_label = "crm"
        verbose_name = _('Position')
        verbose_name_plural = _('Positions')


class SalesContractPosition(Position):
    contract = models.ForeignKey(SalesContract, verbose_name=_("Contract"))

    class Meta:
        app_label = "crm"
        verbose_name = _('Salescontract Position')
        verbose_name_plural = _('Salescontract Positions')

    def __str__(self):
        return _("Salescontract Position") + ": " + str(self.id)


class PurchaseOrderPosition(Position):
    contract = models.ForeignKey(PurchaseOrder, verbose_name=_("Contract"))

    class Meta:
        app_label = "crm"
        verbose_name = _('Purchaseorder Position')
        verbose_name_plural = _('Purchaseorder Positions')

    def __str__(self):
        return _("Purchaseorder Position") + ": " + str(self.id)


class PhoneAddressForContact(PhoneAddress):
    purpose = models.CharField(verbose_name=_("Purpose"), max_length=1, choices=PURPOSESADDRESSINCUSTOMER)
    person = models.ForeignKey(Contact)

    class Meta:
        app_label = "crm"
        verbose_name = _('Phone Address For Contact')
        verbose_name_plural = _('Phone Address For Contact')

    def __str__(self):
        return str(self.phone)


class EmailAddressForContact(EmailAddress):
    purpose = models.CharField(verbose_name=_("Purpose"), max_length=1, choices=PURPOSESADDRESSINCUSTOMER)
    person = models.ForeignKey(Contact)

    class Meta:
        app_label = "crm"
        verbose_name = _('Email Address For Contact')
        verbose_name_plural = _('Email Address For Contact')

    def __str__(self):
        return str(self.email)


class PostalAddressForContact(PostalAddress):
    purpose = models.CharField(verbose_name=_("Purpose"), max_length=1, choices=PURPOSESADDRESSINCUSTOMER)
    person = models.ForeignKey(Contact)

    class Meta:
        app_label = "crm"
        verbose_name = _('Postal Address For Contact')
        verbose_name_plural = _('Postal Address For Contact')

    def __str__(self):
        return xstr(self.prename) + ' ' + xstr(self.name) + ' ' + xstr(self.addressline1)


class PostalAddressForContract(PostalAddress):
    purpose = models.CharField(verbose_name=_("Purpose"), max_length=1, choices=PURPOSESADDRESSINCONTRACT)
    contract = models.ForeignKey(Contract)

    class Meta:
        app_label = "crm"
        verbose_name = _('Postal Address For Contracts')
        verbose_name_plural = _('Postal Address For Contracts')

    def __str__(self):
        return xstr(self.prename) + ' ' + xstr(self.name) + ' ' + xstr(self.addressline1)


class PostalAddressForPurchaseOrder(PostalAddress):
    purpose = models.CharField(verbose_name=_("Purpose"), max_length=1, choices=PURPOSESADDRESSINCONTRACT)
    contract = models.ForeignKey(PurchaseOrder)

    class Meta:
        app_label = "crm"
        verbose_name = _('Postal Address For Contracts')
        verbose_name_plural = _('Postal Address For Contracts')

    def __str__(self):
        return xstr(self.prename) + ' ' + xstr(self.name) + ' ' + xstr(self.addressline1)


class PostalAddressForSalesContract(PostalAddress):
    purpose = models.CharField(verbose_name=_("Purpose"), max_length=1, choices=PURPOSESADDRESSINCONTRACT)
    contract = models.ForeignKey(SalesContract)

    class Meta:
        app_label = "crm"
        verbose_name = _('Postal Address For Contracts')
        verbose_name_plural = _('Postal Address For Contracts')

    def __str__(self):
        return xstr(self.prename) + ' ' + xstr(self.name) + ' ' + xstr(self.addressline1)


class PhoneAddressForContract(PhoneAddress):
    purpose = models.CharField(verbose_name=_("Purpose"), max_length=1, choices=PURPOSESADDRESSINCONTRACT)
    contract = models.ForeignKey(Contract)

    class Meta:
        app_label = "crm"
        verbose_name = _('Phone Address For Contracts')
        verbose_name_plural = _('Phone Address For Contracts')

    def __str__(self):
        return str(self.phone)


class PhoneAddressForSalesContract(PhoneAddress):
    purpose = models.CharField(verbose_name=_("Purpose"), max_length=1, choices=PURPOSESADDRESSINCONTRACT)
    contract = models.ForeignKey(SalesContract)

    class Meta:
        app_label = "crm"
        verbose_name = _('Phone Address For Contracts')
        verbose_name_plural = _('Phone Address For Contracts')

    def __str__(self):
        return str(self.phone)


class PhoneAddressForPurchaseOrder(PhoneAddress):
    purpose = models.CharField(verbose_name=_("Purpose"), max_length=1, choices=PURPOSESADDRESSINCONTRACT)
    contract = models.ForeignKey(PurchaseOrder)

    class Meta:
        app_label = "crm"
        verbose_name = _('Phone Address For Contracts')
        verbose_name_plural = _('Phone Address For Contracts')

    def __str__(self):
        return str(self.phone)


class EmailAddressForContract(EmailAddress):
    purpose = models.CharField(verbose_name=_("Purpose"), max_length=1, choices=PURPOSESADDRESSINCONTRACT)
    contract = models.ForeignKey(Contract)

    class Meta:
        app_label = "crm"
        verbose_name = _('Email Address For Contracts')
        verbose_name_plural = _('Email Address For Contracts')

    def __str__(self):
        return str(self.email)


class EmailAddressForSalesContract(EmailAddress):
    purpose = models.CharField(verbose_name=_("Purpose"), max_length=1, choices=PURPOSESADDRESSINCONTRACT)
    contract = models.ForeignKey(SalesContract)

    class Meta:
        app_label = "crm"
        verbose_name = _('Email Address For Contracts')
        verbose_name_plural = _('Email Address For Contracts')

    def __str__(self):
        return str(self.email)


class EmailAddressForPurchaseOrder(EmailAddress):
    purpose = models.CharField(verbose_name=_("Purpose"), max_length=1, choices=PURPOSESADDRESSINCONTRACT)
    contract = models.ForeignKey(PurchaseOrder)

    class Meta:
        app_label = "crm"
        verbose_name = _('Email Address For Contracts')
        verbose_name_plural = _('Email Address For Contracts')

    def __str__(self):
        return str(self.email)



# -*- coding: utf-8 -*-

from django.contrib import admin
from django.utils.translation import ugettext as _
from koalixcrm.djangoUserExtension.models import *


class InlineUserExtensionPostalAddress(admin.StackedInline):
    model = UserExtensionPostalAddress
    extra = 1
    classes = ('collapse-open',)
    fieldsets = (
        (_('Basics'), {
            'fields': (
            'prefix', 'prename', 'name', 'addressline1', 'addressline2', 'addressline3', 'addressline4', 'zipcode',
            'town', 'state', 'country', 'purpose')
        }),
    )
    allow_add = True


class InlineUserExtensionPhoneAddress(admin.StackedInline):
    model = UserExtensionPhoneAddress
    extra = 1
    classes = ('collapse-open',)
    fieldsets = (
        (_('Basics'), {
            'fields': ('phone', 'purpose',)
        }),
    )
    allow_add = True


class InlineUserExtensionEmailAddress(admin.StackedInline):
    model = UserExtensionEmailAddress
    extra = 1
    classes = ('collapse-open',)
    fieldsets = (
        (_('Basics'), {
            'fields': ('email', 'purpose',)
        }),
    )
    allow_add = True


class OptionUserExtension(admin.ModelAdmin):
    list_display = ('id', 'user', 'defaultTemplateSet', 'defaultCurrency')
    list_display_links = ('id', 'user')
    list_filter = ('user', 'defaultTemplateSet',)
    ordering = ('id',)
    search_fields = ('id', 'user')
    fieldsets = (
        (_('Basics'), {
            'fields': ('user', 'defaultTemplateSet', 'defaultCurrency')
        }),
    )
    save_as = True
    inlines = [InlineUserExtensionPostalAddress, InlineUserExtensionPhoneAddress, InlineUserExtensionEmailAddress]


class OptionTemplateSet(admin.ModelAdmin):
    list_display = ('id', 'title')
    list_display_links = ('id', 'title')
    ordering = ('id',)
    search_fields = ('id', 'title', 'organisationname', 'invoiceXSLFile', 'quoteXSLFile', 'purchaseconfirmationXSLFile',
                     'deilveryorderXSLFile', 'profitLossStatementXSLFile', 'balancesheetXSLFile',
                     'purchaseorderXSLFile',
                     'logo', 'footerTextsalesorders', 'headerTextsalesorders',
                     'headerTextpurchaseorders', 'footerTextpurchaseorders', 'pagefooterleft', 'pagefootermiddle',
                     'bankingaccountref', 'addresser'
                     )
    fieldsets = (
        (_('Basics'), {
            'fields': ('title', 'organisationname', 'invoiceXSLFile', 'quoteXSLFile', 'purchaseconfirmationXSLFile',
                       'deilveryorderXSLFile', 'profitLossStatementXSLFile', 'balancesheetXSLFile',
                       'purchaseorderXSLFile',
                       'logo', 'fopConfigurationFile', 'footerTextsalesorders', 'headerTextsalesorders',
                       'headerTextpurchaseorders', 'footerTextpurchaseorders', 'pagefooterleft', 'pagefootermiddle',
                       'bankingaccountref', 'addresser')
        }),
    )


class OptionXSLFile(admin.ModelAdmin):
    list_display = ('id', 'title')
    list_display_links = ('id', 'title')
    ordering = ('id',)
    fieldsets = (
        (_('Basics'), {
            'fields': ('title', 'xslfile',)
        }),
    )
    allow_add = True


admin.site.register(UserExtension, OptionUserExtension)
admin.site.register(TemplateSet, OptionTemplateSet)
admin.site.register(XSLFile, OptionXSLFile)



# -*- coding: utf-8 -*-
from django import forms
from django.contrib import admin
from django.utils.translation import ugettext as _
from koalixcrm.accounting.views import *


class AccountingPeriodBooking(admin.TabularInline):
    model = Booking
    extra = 1
    show_change_link = True
    can_delete = True
    classes = ['collapse']
    fields = ('fromAccount', 'toAccount', 'description', 'amount', 'bookingDateOnly', 'staff', 'bookingReference',)
    readonly_fields = (
    'fromAccount', 'toAccount', 'description', 'amount', 'bookingDateOnly', 'staff', 'bookingReference',)
    allow_add = True


class OptionBooking(admin.ModelAdmin):
    list_display = ('fromAccount', 'toAccount', 'amount', 'bookingDateOnly', 'staff')
    fieldsets = ((_('Basic'), {'fields': (
    'fromAccount', 'toAccount', 'amount', 'bookingDate', 'staff', 'description', 'bookingReference',
    'accountingPeriod')}),)
    save_as = True

    def save_model(self, request, obj, form, change):
        if (change == True):
            obj.lastmodifiedby = request.user
        else:
            obj.lastmodifiedby = request.user
            obj.staff = request.user
        obj.save()


class AccountForm(forms.ModelForm):
    """ AccountForm is used to overwrite the clean method of the 
    original form and to add an additional checks to the model"""

    class Meta:
        model = Account
        fields = '__all__'

    def clean(self):
        super(AccountForm, self).clean()
        errors = []
        if (self.cleaned_data['isopenreliabilitiesaccount']):
            openliabilitiesaccount = Account.objects.filter(isopenreliabilitiesaccount=True)
            if (self.cleaned_data['accountType'] != "L"):
                errors.append(_('The open liabilites account must be a liabities account'))
            elif openliabilitiesaccount:
                errors.append(_('There may only be one open liablities account in the system'))
        if (self.cleaned_data['isopeninterestaccount']):
            openinterestaccounts = Account.objects.filter(isopeninterestaccount=True)
            if (self.cleaned_data['accountType'] != "A"):
                errors.append(_('The open intrests account must be an asset account'))
            elif openinterestaccounts:
                errors.append(_('There may only be one open intrests account in the system'))
        if (self.cleaned_data['isACustomerPaymentAccount']):
            if (self.cleaned_data['accountType'] != "A"):
                errors.append(_('A customer payment account must be an asset account'))
        if (self.cleaned_data['isProductInventoryActiva']):
            if (self.cleaned_data['accountType'] != "A"):
                errors.append(_('A product inventory account must be an asset account'))
        if len(errors) > 0:
            raise forms.ValidationError(errors)
        return self.cleaned_data


class OptionAccount(admin.ModelAdmin):
    list_display = ('accountNumber', 'accountType', 'title', 'sumOfAllBookings')
    list_display_links = ('accountNumber', 'accountType', 'title', 'sumOfAllBookings')
    fieldsets = ((_('Basic'), {'fields': (
    'accountNumber', 'accountType', 'title', 'description', 'isopenreliabilitiesaccount', 'isopeninterestaccount',
    'isProductInventoryActiva', 'isACustomerPaymentAccount')}),)
    save_as = True

    form = AccountForm


class AccountingPeriodForm(forms.ModelForm):
    """ AccountingPeriodForm is used to overwrite the clean method of the 
    original form and to add an additional check to the model"""

    class Meta:
        model = AccountingPeriod
        fields = '__all__'

    def clean(self):
        super(AccountingPeriodForm, self).clean()
        errors = []
        try:
            if self.cleaned_data['begin'] > self.cleaned_data['end']:
                errors.append(_('The begin date cannot be later than the end date.'))
        except KeyError:
            errors.append(_('The begin and the end date may not be empty'))
        if errors:
            raise forms.ValidationError(errors)
        return self.cleaned_data


class OptionAccountingPeriod(admin.ModelAdmin):
    list_display = ('title', 'begin', 'end')
    list_display_links = ('title', 'begin', 'end')
    fieldsets = (
        (_('Basics'), {
            'fields': ('title', 'begin', 'end')
        }),
    )
    inlines = [AccountingPeriodBooking, ]
    save_as = True

    form = AccountingPeriodForm

    def save_formset(self, request, form, formset, change):
        instances = formset.save(commit=False)
        for instance in instances:
            if (change == True):
                instance.lastmodifiedby = request.user
            else:
                instance.lastmodifiedby = request.user
                instance.staff = request.user
            instance.save()

    def createBalanceSheet(self, request, queryset):
        for obj in queryset:
            response = exportPDF(self, request, obj, "balanceSheet", "/admin/accounting/accountingperiod/")
            return response

    createBalanceSheet.short_description = _("Create PDF of Balance Sheet")

    def createProfitLossStatement(self, request, queryset):
        for obj in queryset:
            response = exportPDF(self, request, obj, "profitLossStatement", "/admin/accounting/accountingperiod/")
            return response

    createProfitLossStatement.short_description = _("Create PDF of Profit Loss Statement Sheet")

    def exportAllAccounts(self, request, queryset):
        for obj in queryset:
            response = exportXML(self, request, obj, "allAccount", "/admin/accounting/accountingperiod/")
            return response

    exportAllAccounts.short_description = _("Create XML of all Accounts")

    actions = ['createBalanceSheet', 'createProfitLossStatement', 'exportAllAccounts', ]


class OptionProductCategorie(admin.ModelAdmin):
    list_display = ('title', 'profitAccount', 'lossAccount')
    list_display_links = ('title', 'profitAccount', 'lossAccount')
    fieldsets = (
        (_('Basics'), {
            'fields': ('title', 'profitAccount', 'lossAccount')
        }),
    )
    save_as = True


admin.site.register(Account, OptionAccount)
admin.site.register(Booking, OptionBooking)
admin.site.register(ProductCategorie, OptionProductCategorie)
admin.site.register(AccountingPeriod, OptionAccountingPeriod)



# -*- coding: utf-8 -*-
from os import path
from subprocess import *
from wsgiref.util import FileWrapper

from django.http import Http404
from django.http import HttpResponse
from django.http import HttpResponseRedirect
from django.utils.translation import ugettext as _
from koalixcrm.crm.exceptions import TemplateSetMissing
from koalixcrm.crm.exceptions import UserExtensionMissing
from koalixcrm.crm.models import *


def exportPDF(callingModelAdmin, request, whereToCreateFrom, whatToCreate, redirectTo):
    """This method exports PDFs provided by different Models in the crm application

        Args:
          callingModelAdmin (ModelAdmin):  The calling ModelAdmin must be provided for error message response.
          request: The request User is to know where to save the error message
          whereToCreateFrom (Model):  The model from which a PDF should be exported
          whatToCreate (str): What document Type that has to be
          redirectTo (str): String that describes to where the method sould redirect in case of an error

        Returns:
              HTTpResponse with a PDF when successful
              HTTpResponseRedirect when not successful

        Raises:
          raises Http404 exception if anything goes wrong"""
    try:
        pdf = whereToCreateFrom.createPDF(whatToCreate)
        response = HttpResponse(FileWrapper(open(pdf, 'rb')), content_type='application/pdf')
        response['Content-Length'] = path.getsize(pdf)
    except (TemplateSetMissing, UserExtensionMissing, CalledProcessError) as e:
        if type(e) == UserExtensionMissing:
            response = HttpResponseRedirect(redirectTo)
            callingModelAdmin.message_user(request, _("User Extension Missing"))
        elif type(e) == TemplateSetMissing:
            response = HttpResponseRedirect(redirectTo)
            callingModelAdmin.message_user(request, _("Templateset Missing"))
        elif type(e) == CalledProcessError:
            response = HttpResponseRedirect(redirectTo)
            callingModelAdmin.message_user(request, e.output)
        else:
            raise Http404
    return response


def selectaddress(invoiceid):
    invoice = Invoice.objects.get(id=invoiceid)
    address = invoice.contract



# -*- coding: utf-8 -*-

from os import mkdir
from os import path
from shutil import copy

import crm
import djangoUserExtension
from django.contrib.auth.models import User
from django.core.management.base import BaseCommand
from django.utils.translation import ugettext as _
from filebrowser.settings import DIRECTORY
from filebrowser.settings import MEDIA_ROOT
from settings import PROJECT_ROOT

DEFAULT_FILE = 'dashboard.py'


class Command(BaseCommand):
    help = (
    'This Command is going to install the default Templates, given by the koalixcrm base installation, in your django instance. Be sure you first run syncdb')
    args = '[]'
    label = 'application name'

    def handle(self, **options):
        invoicetemplate = 'invoice.xsl'
        quotetemplate = 'quote.xsl'
        deliveryordertemplate = 'deliveryorder.xsl'
        purchaseordertemplate = 'purchaseorder.xsl'
        purchaseconfirmationtemplate = 'purchaseconfirmation.xsl'
        balancesheettemplate = 'balancesheet.xsl'
        profitlossstatementtemplate = 'profitlossstatement.xsl'
        listoftemplatefiles = {'invoice': invoicetemplate,
                               'quote': quotetemplate,
                               'deliveryorder': deliveryordertemplate,
                               'purchaseconfirmation': purchaseconfirmationtemplate,
                               'purchaseorder': purchaseordertemplate,
                               'balancesheet': balancesheettemplate,
                               'profitlossstatement': profitlossstatementtemplate,
                               }

        configfile = 'fontconfig.xml'
        dejavusansfile = 'dejavusans-bold.xml'
        dejavusansboldfile = 'dejavusans.xml'
        logo = 'logo.jpg'
        copy('templatefiles/generic/' + configfile, MEDIA_ROOT + DIRECTORY + 'templatefiles/' + configfile)
        copy('templatefiles/generic/' + logo, MEDIA_ROOT + DIRECTORY + 'templatefiles/' + logo)
        copy('templatefiles/generic/' + dejavusansfile, MEDIA_ROOT + DIRECTORY + 'templatefiles/' + dejavusansfile)
        copy('templatefiles/generic/' + dejavusansboldfile,
             MEDIA_ROOT + DIRECTORY + 'templatefiles/' + dejavusansboldfile)
        listofadditionalfiles = ('dejavusans-bold.xml', 'dejavusans.xml',)
        if path.exists('templatefiles'):
            templateset = djangoUserExtension.models.TemplateSet()
            templateset.title = 'defaultTemplateSet'
            if (path.exists(MEDIA_ROOT + DIRECTORY + 'templatefiles') == False):
                mkdir(MEDIA_ROOT + DIRECTORY + 'templatefiles')
            for template in listoftemplatefiles:
                if path.exists(PROJECT_ROOT + 'templatefiles/en/' + listoftemplatefiles[template]):
                    copy('templatefiles/en/' + listoftemplatefiles[template],
                         MEDIA_ROOT + DIRECTORY + 'templatefiles/' + listoftemplatefiles[template])
                    xslfile = djangoUserExtension.models.XSLFile()
                    xslfile.title = template
                    xslfile.xslfile = DIRECTORY + 'templatefiles/' + listoftemplatefiles[template]
                    xslfile.save()
                    if template == 'invoice':
                        templateset.invoiceXSLFile = xslfile
                    elif template == 'quote':
                        templateset.quoteXSLFile = xslfile
                    elif template == 'purchaseconfirmation':
                        templateset.purchaseconfirmationXSLFile = xslfile
                    elif template == 'purchaseorder':
                        templateset.purchaseorderXSLFile = xslfile
                    elif template == 'deliveryorder':
                        templateset.deilveryorderXSLFile = xslfile
                    elif template == 'profitlossstatement':
                        templateset.profitLossStatementXSLFile = xslfile
                    elif template == 'balancesheet':
                        templateset.balancesheetXSLFile = xslfile
                    print(listoftemplatefiles[template])
                else:
                    print(listoftemplatefiles)
                    print(listoftemplatefiles[template])
                    print(template)
                    print(MEDIA_ROOT + DIRECTORY + 'templatefiles/' + listoftemplatefiles[template])
                    raise FileNotFoundException
            templateset.logo = DIRECTORY + 'templatefiles/' + logo
            templateset.bankingaccountref = "xx-xxxxxx-x"
            templateset.addresser = _("John Smit, Sample Company, 8976 Smallville")
            templateset.fopConfigurationFile = DIRECTORY + 'templatefiles/' + configfile
            templateset.headerTextsalesorders = _(
                "According to your wishes the contract consists of the following positions:")
            templateset.footerTextsalesorders = _("Thank you for your interest in our company \n Best regards")
            templateset.headerTextpurchaseorders = _("We would like to order the following positions:")
            templateset.footerTextpurchaseorders = _("Best regards")
            templateset.pagefooterleft = _("Sample Company")
            templateset.pagefootermiddle = _("Sample Address")
            templateset.save()
            currency = crm.models.Currency()
            currency.description = "US Dollar"
            currency.shortName = "USD"
            currency.rounding = "0.10"
            currency.save()
            userExtension = djangoUserExtension.models.UserExtension()
            userExtension.defaultTemplateSet = templateset
            userExtension.defaultCurrency = currency
            userExtension.user = User.objects.all()[0]
            userExtension.save()
            postaladdress = djangoUserExtension.models.UserExtensionPostalAddress()
            postaladdress.purpose = 'H'
            postaladdress.name = "John"
            postaladdress.prename = "Smith"
            postaladdress.addressline1 = "Ave 1"
            postaladdress.zipcode = 899887
            postaladdress.town = "Smallville"
            postaladdress.userExtension = userExtension
            postaladdress.save()
            phoneaddress = djangoUserExtension.models.UserExtensionPhoneAddress()
            phoneaddress.phone = "1293847"
            phoneaddress.purpose = 'H'
            phoneaddress.userExtension = userExtension
            phoneaddress.save()
            emailaddress = djangoUserExtension.models.UserExtensionEmailAddress()
            emailaddress.email = "john.smith@smallville.com"
            emailaddress.purpose = 'H'
            emailaddress.userExtension = userExtension
            emailaddress.save()

            for additionalfile in listofadditionalfiles:
                if path.exists('templatefiles' + additionalfile):
                    shutil.copy('templatefiles' + additionalfile, DIRECTORY + 'templatefiles/')



# -*- coding: utf-8 -*-

from django.db import models
from django.utils.translation import ugettext as _
from filebrowser.fields import FileBrowseField
from koalixcrm.crm import models as crmmodels
from koalixcrm.djangoUserExtension.const.purpose import *


class XSLFile(models.Model):
    title = models.CharField(verbose_name=_("Title"), max_length=100, blank=True, null=True)
    xslfile = FileBrowseField(verbose_name=_("XSL File"), max_length=200)

    class Meta:
        app_label = "djangoUserExtension"
        verbose_name = _('XSL File')
        verbose_name_plural = _('XSL Files')

    def __str__(self):
        return str(self.id) + ' ' + self.title


class UserExtension(models.Model):
    user = models.ForeignKey('auth.User')
    defaultTemplateSet = models.ForeignKey('TemplateSet')
    defaultCurrency = models.ForeignKey('crm.Currency')

    class Meta:
        app_label = "djangoUserExtension"
        verbose_name = _('User Extention')
        verbose_name_plural = _('User Extentions')

    def __str__(self):
        return str(self.id) + ' ' + self.user.__str__()


class TemplateSet(models.Model):
    organisationname = models.CharField(verbose_name=_("Name of the Organisation"), max_length=200)
    title = models.CharField(verbose_name=_("Title"), max_length=100)
    invoiceXSLFile = models.ForeignKey(XSLFile, verbose_name=_("XSL File for Invoice"),
                                       related_name="db_reltemplateinvoice")
    quoteXSLFile = models.ForeignKey(XSLFile, verbose_name=_("XSL File for Quote"), related_name="db_reltemplatequote")
    purchaseorderXSLFile = models.ForeignKey(XSLFile, verbose_name=_("XSL File for Purchaseorder"),
                                             related_name="db_reltemplatepurchaseorder")
    purchaseconfirmationXSLFile = models.ForeignKey(XSLFile, verbose_name=_("XSL File for Purchase Confirmation"),
                                                    related_name="db_reltemplatepurchaseconfirmation")
    deilveryorderXSLFile = models.ForeignKey(XSLFile, verbose_name=_("XSL File for Deilvery Order"),
                                             related_name="db_reltemplatedeliveryorder")
    profitLossStatementXSLFile = models.ForeignKey(XSLFile, verbose_name=_("XSL File for Profit Loss Statement"),
                                                   related_name="db_reltemplateprofitlossstatement")
    balancesheetXSLFile = models.ForeignKey(XSLFile, verbose_name=_("XSL File for Balancesheet"),
                                            related_name="db_reltemplatebalancesheet")
    logo = FileBrowseField(verbose_name=_("Logo for the PDF generation"), blank=True, null=True, max_length=200)
    bankingaccountref = models.CharField(max_length=60, verbose_name=_("Reference to Banking Account"), blank=True,
                                         null=True)
    addresser = models.CharField(max_length=200, verbose_name=_("Addresser"), blank=True, null=True)
    fopConfigurationFile = FileBrowseField(verbose_name=_("FOP Configuration File"), blank=True, null=True,
                                           max_length=200)
    footerTextsalesorders = models.TextField(verbose_name=_("Footer Text On Salesorders"), blank=True, null=True)
    headerTextsalesorders = models.TextField(verbose_name=_("Header Text On Salesorders"), blank=True, null=True)
    headerTextpurchaseorders = models.TextField(verbose_name=_("Header Text On Purchaseorders"), blank=True, null=True)
    footerTextpurchaseorders = models.TextField(verbose_name=_("Footer Text On Purchaseorders"), blank=True, null=True)
    pagefooterleft = models.CharField(max_length=40, verbose_name=_("Page Footer Left"), blank=True, null=True)
    pagefootermiddle = models.CharField(max_length=40, verbose_name=_("Page Footer Middle"), blank=True, null=True)

    class Meta:
        app_label = "djangoUserExtension"
        verbose_name = _('Templateset')
        verbose_name_plural = _('Templatesets')

    def __str__(self):
        return str(self.id) + ' ' + self.title


class UserExtensionPostalAddress(crmmodels.PostalAddress):
    purpose = models.CharField(verbose_name=_("Purpose"), max_length=1, choices=PURPOSESADDRESSINUSEREXTENTION)
    userExtension = models.ForeignKey(UserExtension)

    def __str__(self):
        return self.name + ' ' + self.prename

    class Meta:
        app_label = "djangoUserExtension"
        # app_label_koalix = _('Djang User Extention')
        verbose_name = _('Postal Address for User Extention')
        verbose_name_plural = _('Postal Address for User Extention')


class UserExtensionPhoneAddress(crmmodels.PhoneAddress):
    purpose = models.CharField(verbose_name=_("Purpose"), max_length=1, choices=PURPOSESADDRESSINUSEREXTENTION)
    userExtension = models.ForeignKey(UserExtension)

    def __str__(self):
        return self.phone

    class Meta:
        app_label = "djangoUserExtension"
        # app_label_koalix = _('Djang User Extention')
        verbose_name = _('Phonenumber for User Extention')
        verbose_name_plural = _('Phonenumber for User Extention')


class UserExtensionEmailAddress(crmmodels.EmailAddress):
    purpose = models.CharField(verbose_name=_("Purpose"), max_length=1, choices=PURPOSESADDRESSINUSEREXTENTION)
    userExtension = models.ForeignKey(UserExtension)

    def __str__(self):
        return self.email

    class Meta:
        app_label = "djangoUserExtension"
        # app_label_koalix = _('Djang User Extention')
        verbose_name = _('Email Address for User Extention')
        verbose_name_plural = _('Email Address for User Extention')



# -*- coding: utf-8 -*-

from datetime import *
from subprocess import *
from xml.dom.minidom import Document

from django.conf import settings
from django.core import serializers
from django.db import models
from django.utils.translation import ugettext as _
from koalixcrm.accounting.const.accountTypeChoices import *
from koalixcrm.accounting.exceptions import NoObjectsToBeSerialzed
from koalixcrm.accounting.exceptions import ProgrammingError
from koalixcrm.crm.exceptions import UserExtensionMissing


class AccountingPeriod(models.Model):
    title = models.CharField(max_length=200, verbose_name=_("Title"))  # For example "Year 2009", "1st Quarter 2009"
    begin = models.DateField(verbose_name=_("Begin"))
    end = models.DateField(verbose_name=_("End"))

    @staticmethod
    def getCurrentValidAccountingPeriod():
        """Returns the accounting period that is currently valid. Valid is an accountingPeriod when the current date
          lies between begin and end of the accountingPeriod

        Args:
          no arguments

        Returns:
          accoutingPeriod (AccoutingPeriod)

        Raises:
          NoFeasableAccountingPeriodFound when there is no valid accounting Period"""
        currentValidAccountingPeriod = None
        for accountingPeriod in AccountingPeriod.objects.all():
            if accountingPeriod.begin < date.today() and accountingPeriod.end > date.today():
                return accountingPeriod
        if currentValidAccountingPeriod == None:
            raise NoFeasableAccountingPeriodFound()

    @staticmethod
    def getAllPriorAccountingPeriods(targetAccountingPeriod):
        """Returns the accounting period that is currently valid. Valid is an accountingPeriod when the current date
          lies between begin and end of the accountingPeriod

        Args:
          no arguments

        Returns:
          accoutingPeriods (List of AccoutingPeriod)

        Raises:
          NoPriorAccountingPeriodFound when there is no valid accounting Period"""
        currentValidAccountingPeriod = None
        accountingPeriods = []
        for accountingPeriod in AccountingPeriod.objects.all():
            if accountingPeriod.end < targetAccountingPeriod.begin:
                accountingPeriods.append(accountingPeriod)
        if accountingPeriods == []:
            raise NoPriorAccountingPeriodFound()
        return accountingPeriods

    def createXML(self, raisedbyuser, whatToCreate):
        """This method serialize requestd objects into a XML file which is located in the PDF_OUTPUT_ROOT folder.

          Args:
            whatToCreate (str): Which objects that have to be serialized

          Returns:
            path_full to the location of the file

          Raises:
            ProgrammingError will be raised when incorrect objects to be serialized was selected
            NoObjectToBeSerialized will be raised when no object can be serialized"""

        XMLSerializer = serializers.get_serializer("xml")
        xml_serializer = XMLSerializer()
        path_fullToOutputFile = " "
        if whatToCreate == "allAccount":
            path_fullToOutputFile = settings.PDF_OUTPUT_ROOT + "accounts.xml"
            objectsToSerialize = Account.objects.all()
        else:
            raise ProgrammingError(
                _("During XML Export it was not correctly specified which data that has to be exported"))
        out = open(settings.PDF_OUTPUT_ROOT + "accounts.xml", "w")
        if objectsToSerialize == '':
            raise NoObjectsToBeSerialzed(_("During XML Export it was not correctly specied data has to be exported"))
        else:
            xml_serializer.serialize(objectsToSerialize, stream=out, indent=3)
        out.close()
        return path_fullToOutputFile

        # TODO  def importAllAccountsXML(self):

    def createPDF(self, raisedbyuser, whatToCreate):
        userExtension = djangoUserExtension.models.UserExtension.objects.filter(user=raisedbyuser.id)
        if (len(userExtension) == 0):
            raise UserExtensionMissing(_("During BalanceSheet PDF Export"))
        doc = Document()
        if (whatToCreate == "balanceSheet"):
            main = doc.createElement("koalixaccountingbalacesheet")
            out = open(settings.PDF_OUTPUT_ROOT + "balancesheet_" + str(self.id) + ".xml", "wb")
        else:
            main = doc.createElement("koalixaccountingprofitlossstatement")
            out = open(settings.PDF_OUTPUT_ROOT + "profitlossstatement_" + str(self.id) + ".xml", "wb")
        accountingPeriodName = doc.createElement("accountingPeriodName")
        accountingPeriodName.appendChild(doc.createTextNode(self.__str__()))
        main.appendChild(accountingPeriodName)
        organisiationname = doc.createElement("organisiationname")
        organisiationname.appendChild(doc.createTextNode(userExtension[0].defaultTemplateSet.organisationname))
        main.appendChild(organisiationname)
        accountingPeriodTo = doc.createElement("accountingPeriodTo")
        accountingPeriodTo.appendChild(doc.createTextNode(self.end.year.__str__()))
        main.appendChild(accountingPeriodTo)
        accountingPeriodFrom = doc.createElement("accountingPeriodFrom")
        accountingPeriodFrom.appendChild(doc.createTextNode(self.begin.year.__str__()))
        main.appendChild(accountingPeriodFrom)
        headerPicture = doc.createElement("headerpicture")
        headerPicture.appendChild(doc.createTextNode(userExtension[0].defaultTemplateSet.logo.path_full))
        main.appendChild(headerPicture)
        accountNumber = doc.createElement("AccountNumber")
        accounts = Account.objects.all()
        overallValueBalance = 0
        overallValueProfitLoss = 0
        for account in list(accounts):
            withinAccountingPeriod = account.sumOfAllBookingsWithinAccountingPeriod(self)
            beforeAccountingPeriod = account.sumOfAllBookingsBeforeAccountingPeriod(self)
            currentValue = withinAccountingPeriod + beforeAccountingPeriod
            if (currentValue != 0):
                currentAccountElement = doc.createElement("Account")
                accountNumber = doc.createElement("AccountNumber")
                accountNumber.appendChild(doc.createTextNode(account.accountNumber.__str__()))
                beforeAccountingPeriodAccountElement = doc.createElement("beforeAccountingPeriod")
                beforeAccountingPeriodAccountElement.appendChild(doc.createTextNode(beforeAccountingPeriod.__str__()))
                currentValueElement = doc.createElement("currentValue")
                currentValueElement.appendChild(doc.createTextNode(currentValue.__str__()))
                accountNameElement = doc.createElement("accountName")
                accountNameElement.appendChild(doc.createTextNode(account.title))
                currentAccountElement.setAttribute("accountType", account.accountType.__str__())
                currentAccountElement.appendChild(accountNumber)
                currentAccountElement.appendChild(accountNameElement)
                currentAccountElement.appendChild(currentValueElement)
                currentAccountElement.appendChild(beforeAccountingPeriodAccountElement)
                main.appendChild(currentAccountElement)
                if account.accountType == "A":
                    overallValueBalance = overallValueBalance + currentValue;
                if account.accountType == "L":
                    overallValueBalance = overallValueBalance - currentValue;
                if account.accountType == "E":
                    overallValueProfitLoss = overallValueProfitLoss + currentValue;
                if account.accountType == "S":
                    overallValueProfitLoss = overallValueProfitLoss - currentValue;
        totalProfitLoss = doc.createElement("TotalProfitLoss")
        totalProfitLoss.appendChild(doc.createTextNode(overallValueProfitLoss.__str__()))
        main.appendChild(totalProfitLoss)
        totalBalance = doc.createElement("TotalBalance")
        totalBalance.appendChild(doc.createTextNode(overallValueBalance.__str__()))
        main.appendChild(totalBalance)
        doc.appendChild(main)
        out.write(doc.toprettyxml(indent=" ", newl="\n", encoding="utf-8"))
        out.close()
        if (whatToCreate == "balanceSheet"):
            check_output(
                ['/usr/bin/fop', '-c', userExtension[0].defaultTemplateSet.fopConfigurationFile.path_full, '-xml',
                 settings.PDF_OUTPUT_ROOT + 'balancesheet_' + str(self.id) + '.xml', '-xsl',
                 userExtension[0].defaultTemplateSet.balancesheetXSLFile.xslfile.path_full, '-pdf',
                 settings.PDF_OUTPUT_ROOT + 'balancesheet_' + str(self.id) + '.pdf'], stderr=STDOUT)
            return settings.PDF_OUTPUT_ROOT + "balancesheet_" + str(self.id) + ".pdf"
        else:
            check_output(
                ['/usr/bin/fop', '-c', userExtension[0].defaultTemplateSet.fopConfigurationFile.path_full, '-xml',
                 settings.PDF_OUTPUT_ROOT + 'profitlossstatement_' + str(self.id) + '.xml', '-xsl',
                 userExtension[0].defaultTemplateSet.profitLossStatementXSLFile.xslfile.path_full, '-pdf',
                 settings.PDF_OUTPUT_ROOT + 'profitlossstatement_' + str(self.id) + '.pdf'], stderr=STDOUT)
            return settings.PDF_OUTPUT_ROOT + "profitlossstatement_" + str(self.id) + ".pdf"

    def __str__(self):
        return self.title

        # TODO: def createNewAccountingPeriod() Neues Geschäftsjahr erstellen

    class Meta:
        app_label = "accounting"
        verbose_name = _('Accounting Period')
        verbose_name_plural = _('Accounting Periods')


class Account(models.Model):
    accountNumber = models.IntegerField(verbose_name=_("Account Number"))
    title = models.CharField(verbose_name=_("Account Title"), max_length=50)
    accountType = models.CharField(verbose_name=_("Account Type"), max_length=1, choices=ACCOUNTTYPECHOICES)
    description = models.TextField(verbose_name=_("Description"), null=True, blank=True)
    isopenreliabilitiesaccount = models.BooleanField(verbose_name=_("Is The Open Liabilities Account"))
    isopeninterestaccount = models.BooleanField(verbose_name=_("Is The Open Interests Account"))
    isProductInventoryActiva = models.BooleanField(verbose_name=_("Is a Product Inventory Account"))
    isACustomerPaymentAccount = models.BooleanField(verbose_name=_("Is a Customer Payment Account"))

    def sumOfAllBookings(self):
        sum = self.allBookings(fromAccount=False) - self.allBookings(fromAccount=True)
        if (self.accountType == 'S' or self.accountType == 'L'):
            sum = 0 - sum
        return sum

    sumOfAllBookings.short_description = _("Value");

    def sumOfAllBookingsWithinAccountingPeriod(self, accountingPeriod):
        sum = self.allBookingsInAccountingPeriod(fromAccount=False,
                                                 accountingPeriod=accountingPeriod) - self.allBookingsInAccountingPeriod(
            fromAccount=True, accountingPeriod=accountingPeriod)
        if (self.accountType == 'S' or self.accountType == 'L'):
            sum = 0 - sum
        return sum

    def sumOfAllBookingsBeforeAccountingPeriod(self, currentAccountingPeriod):
        accountingPeriods = AccountingPeriod.getAllPriorAccountingPeriods(currentAccountingPeriod)
        sum = 0
        for accountingPeriod in accountingPeriods:
            sum = sum + self.allBookingsInAccountingPeriod(fromAccount=False,
                                                           accountingPeriod=accountingPeriod) - self.allBookingsInAccountingPeriod(
                fromAccount=True, accountingPeriod=accountingPeriod)
        if (self.accountType == 'S' or self.accountType == 'L'):
            sum = 0 - sum
        return sum

    def allBookings(self, fromAccount):
        sum = 0
        if (fromAccount == True):
            bookings = Booking.objects.filter(fromAccount=self.id)
        else:
            bookings = Booking.objects.filter(toAccount=self.id)

        for booking in list(bookings):
            sum = sum + booking.amount

        return sum

    def allBookingsInAccountingPeriod(self, fromAccount, accountingPeriod):
        sum = 0
        if (fromAccount == True):
            bookings = Booking.objects.filter(fromAccount=self.id, accountingPeriod=accountingPeriod.id)
        else:
            bookings = Booking.objects.filter(toAccount=self.id, accountingPeriod=accountingPeriod.id)

        for booking in list(bookings):
            sum = sum + booking.amount

        return sum

    def __str__(self):
        return self.accountNumber.__str__() + " " + self.title

    class Meta:
        app_label = "accounting"
        verbose_name = _('Account')
        verbose_name_plural = _('Account')
        ordering = ['accountNumber']


class ProductCategorie(models.Model):
    title = models.CharField(verbose_name=_("Product Categorie Title"), max_length=50)
    profitAccount = models.ForeignKey(Account, verbose_name=_("Profit Account"), limit_choices_to={"accountType": "E"},
                                      related_name="db_profit_account")
    lossAccount = models.ForeignKey(Account, verbose_name=_("Loss Account"), limit_choices_to={"accountType": "S"},
                                    related_name="db_loss_account")

    class Meta:
        app_label = "accounting"
        verbose_name = _('Product Categorie')
        verbose_name_plural = _('Product Categories')

    def __str__(self):
        return self.title


class Booking(models.Model):
    fromAccount = models.ForeignKey(Account, verbose_name=_("From Account"), related_name="db_booking_fromaccount")
    toAccount = models.ForeignKey(Account, verbose_name=_("To Account"), related_name="db_booking_toaccount")
    amount = models.DecimalField(max_digits=20, decimal_places=2, verbose_name=_("Amount"))
    description = models.CharField(verbose_name=_("Description"), max_length=120, null=True, blank=True)
    bookingReference = models.ForeignKey('crm.Invoice', verbose_name=_("Booking Reference"), null=True, blank=True)
    bookingDate = models.DateTimeField(verbose_name=_("Booking at"))
    accountingPeriod = models.ForeignKey(AccountingPeriod, verbose_name=_("AccountingPeriod"))
    staff = models.ForeignKey('auth.User', limit_choices_to={'is_staff': True}, blank=True,
                              verbose_name=_("Reference Staff"), related_name="db_booking_refstaff")
    dateofcreation = models.DateTimeField(verbose_name=_("Created at"), auto_now=True)
    lastmodification = models.DateTimeField(verbose_name=_("Last modified"), auto_now_add=True)
    lastmodifiedby = models.ForeignKey('auth.User', limit_choices_to={'is_staff': True}, blank=True,
                                       verbose_name=_("Last modified by"), related_name="db_booking_lstmodified")

    def bookingDateOnly(self):
        return self.bookingDate.date()

    bookingDateOnly.short_description = _("Date");

    def __str__(self):
        return self.fromAccount.__str__() + " " + self.toAccount.__str__() + " " + self.amount.__str__()

    class Meta:
        app_label = "accounting"
        verbose_name = _('Booking')
        verbose_name_plural = _('Bookings')



from setuptools import setup, find_packages

setup(name='koalix-crm',
      version='1.11.dev1',
      description='koalixcrm is a tiny and easy to use Customer-Relationship-Management Software (CRM) including an also tiny and easy to use Accounting Software',
      url='http://github.com/scaphilo/koalixcrm',
      author='Aaron Riedener',
      author_email='aaron.riedener@gmail.com',
      license='BSD',
      packages=find_packages(),
      install_requires=['Django>=1.11.3','django-filebrowser-no-grappelli>=3.7.1','lxml>=3.8.0','olefile>=0.44','Pillow>=4.2.1','psycopg2>=2.7.3', 'pytz>=2017.2'],
      zip_safe=False,
      classifiers=['Development Status :: 4 - Beta','Programming Language :: Python :: 3.4',],
      python_requires='~=3.5',
      include_package_data=True,
)




from datetime import *

from django.db import models
from django.utils.translation import ugettext as _
from filebrowser.fields import FileBrowseField
from koalixcrm.crm import models as crmmodels
from koalixcrm.subscriptions.const.events import *


class Subscription(models.Model):
    contract = models.ForeignKey(crmmodels.Contract, verbose_name=_('Subscription Type'))
    subscriptiontype = models.ForeignKey('SubscriptionType', verbose_name=_('Subscription Type'), null=True)

    def createSubscriptionFromContract(self, contract):
        self.contract = contract
        self.save()
        return self

    def createQuote(self):
        quote = Quote()
        quote.contract = self.contract
        quote.discount = 0
        quote.staff = self.contract.staff
        quote.customer = self.contract.defaultcustomer
        quote.status = 'C'
        quote.currency = self.contract.defaultcurrency
        quote.validuntil = date.today().__str__()
        quote.dateofcreation = date.today().__str__()
        quote.save()
        return quote

    def createInvoice(self):
        invoice = crmmodels.Invoice()
        invoice.contract = self.contract
        invoice.discount = 0
        invoice.staff = self.contract.staff
        invoice.customer = self.contract.defaultcustomer
        invoice.status = 'C'
        invoice.currency = self.contract.defaultcurrency
        invoice.payableuntil = date.today() + timedelta(
            days=self.contract.defaultcustomer.defaultCustomerBillingCycle.timeToPaymentDate)
        invoice.dateofcreation = date.today().__str__()
        invoice.save()
        return invoice

    class Meta:
        app_label = "subscriptions"
        # app_label_koalix = _("Subscriptions")
        verbose_name = _('Subscription')
        verbose_name_plural = _('Subscriptions')


class SubscriptionEvent(models.Model):
    subscriptions = models.ForeignKey('Subscription', verbose_name=_('Subscription'))
    eventdate = models.DateField(verbose_name=_("Event Date"), blank=True, null=True)
    event = models.CharField(max_length=1, choices=SUBSCRITIONEVENTS, verbose_name=_('Event'))

    def __str__(self):
        return self.event

    class Meta:
        app_label = "subscriptions"
        # app_label_koalix = _("Subscriptions")
        verbose_name = _('Subscription Event')
        verbose_name_plural = _('Subscription Events')


class SubscriptionType(crmmodels.Product):
    cancelationPeriod = models.IntegerField(verbose_name=_("Cancelation Period (months)"), blank=True, null=True)
    automaticContractExtension = models.IntegerField(verbose_name=_("Automatic Contract Extension (months)"),
                                                     blank=True, null=True)
    automaticContractExtensionReminder = models.IntegerField(
        verbose_name=_("Automatic Contract Extensoin Reminder (days)"), blank=True, null=True)
    minimumDuration = models.IntegerField(verbose_name=_("Minimum Contract Duration"), blank=True, null=True)
    paymentIntervall = models.IntegerField(verbose_name=_("Payment Intervall (days)"), blank=True, null=True)
    contractDocument = FileBrowseField(verbose_name=_("Contract Documents"), blank=True, null=True, max_length=200)

    class Meta:
        app_label = "subscriptions"
        # app_label_koalix = _("Subscriptions")
        verbose_name = _('Subscription Type')
        verbose_name_plural = _('Subscription Types')



# -*- coding: utf-8 -*-
from django import forms
from django.contrib import admin
from django.contrib.admin import helpers
from django.http import HttpResponseRedirect
from django.shortcuts import render
from django.template.context_processors import csrf
from django.utils.translation import ugettext as _
from koalixcrm.accounting.models import Account
from koalixcrm.accounting.models import Booking
from koalixcrm.crm.views import *
from koalixcrm.plugin import *


class ContractPostalAddress(admin.StackedInline):
    model = PostalAddressForContract
    extra = 1
    classes = ['collapse']
    fieldsets = (
        ('Basics', {
            'fields': (
            'prefix', 'prename', 'name', 'addressline1', 'addressline2', 'addressline3', 'addressline4', 'zipcode',
            'town', 'state', 'country', 'purpose'),
        }),
    )
    allow_add = True


class ContractPhoneAddress(admin.TabularInline):
    model = PhoneAddressForContract
    extra = 1
    classes = ['collapse']
    fieldsets = (
        ('Basics', {
            'fields': ('phone', 'purpose',)
        }),
    )
    allow_add = True


class ContractEmailAddress(admin.TabularInline):
    model = EmailAddressForContract
    extra = 1
    classes = ['collapse']
    fieldsets = (
        ('Basics', {
            'fields': ('email', 'purpose',)
        }),
    )
    allow_add = True


class PurchaseOrderPostalAddress(admin.StackedInline):
    model = PostalAddressForPurchaseOrder
    extra = 1
    classes = ['collapse']
    fieldsets = (
        ('Basics', {
            'fields': (
            'prefix', 'prename', 'name', 'addressline1', 'addressline2', 'addressline3', 'addressline4', 'zipcode',
            'town', 'state', 'country', 'purpose')
        }),
    )
    allow_add = True


class PurchaseOrderPhoneAddress(admin.TabularInline):
    model = PhoneAddressForPurchaseOrder
    extra = 1
    classes = ['collapse']
    fieldsets = (
        ('Basics', {
            'fields': ('phone', 'purpose',)
        }),
    )
    allow_add = True


class PurchaseOrderEmailAddress(admin.TabularInline):
    model = EmailAddressForPurchaseOrder
    extra = 1
    classes = ['collapse']
    fieldsets = (
        ('Basics', {
            'fields': ('email', 'purpose',)
        }),
    )
    allow_add = True


class SalesContractPostalAddress(admin.StackedInline):
    model = PostalAddressForSalesContract
    extra = 1
    classes = ['collapse']
    fieldsets = (
        ('Basics', {
            'fields': (
            'prefix', 'prename', 'name', 'addressline1', 'addressline2', 'addressline3', 'addressline4', 'zipcode',
            'town', 'state', 'country', 'purpose')
        }),
    )
    allow_add = True


class SalesContractPhoneAddress(admin.TabularInline):
    model = PhoneAddressForSalesContract
    extra = 1
    classes = ['collapse']
    fieldsets = (
        ('Basics', {
            'fields': ('phone', 'purpose',)
        }),
    )
    allow_add = True


class SalesContractEmailAddress(admin.TabularInline):
    model = EmailAddressForSalesContract
    extra = 1
    classes = ['collapse']
    fieldsets = (
        ('Basics', {
            'fields': ('email', 'purpose',)
        }),
    )
    allow_add = True


class SalesContractInlinePosition(admin.TabularInline):
    model = SalesContractPosition
    extra = 1
    classes = ['collapse']
    fieldsets = (
        ('', {
            'fields': (
            'positionNumber', 'quantity', 'unit', 'product', 'description', 'discount', 'overwriteProductPrice',
            'positionPricePerUnit', 'sentOn', 'supplier')
        }),
    )
    allow_add = True


class InlineQuote(admin.TabularInline):
    model = Quote
    classes = ['collapse']
    show_change_link = True
    can_delete = True
    extra = 1
    readonly_fields = (
    'description', 'contract', 'customer', 'validuntil', 'status', 'lastPricingDate', 'lastCalculatedPrice',
    'lastCalculatedTax',)
    fieldsets = (
        (_('Basics'), {
            'fields': ('description', 'contract', 'customer', 'validuntil', 'status')
        }),
        (_('Advanced (not editable)'), {
            'classes': ('collapse',),
            'fields': ('lastPricingDate', 'lastCalculatedPrice', 'lastCalculatedTax',)
        }),
    )
    allow_add = False


class InlineInvoice(admin.TabularInline):
    model = Invoice
    classes = ['collapse']
    show_change_link = True
    can_delete = True
    extra = 1
    readonly_fields = (
    'lastPricingDate', 'lastCalculatedPrice', 'lastCalculatedTax', 'description', 'contract', 'customer',
    'payableuntil', 'status')
    fieldsets = (
        (_('Basics'), {
            'fields': ('description', 'contract', 'customer', 'payableuntil', 'status'),
        }),
        (_('Advanced (not editable)'), {
            'classes': ('collapse',),
            'fields': ('lastPricingDate', 'lastCalculatedPrice', 'lastCalculatedTax',)
        }),
    )

    allow_add = False


class InlinePurchaseOrder(admin.TabularInline):
    model = PurchaseOrder
    classes = ['collapse']
    show_change_link = True
    can_delete = True
    extra = 1
    readonly_fields = (
    'description', 'contract', 'supplier', 'externalReference', 'status', 'lastPricingDate', 'lastCalculatedPrice')
    fieldsets = (
        (_('Basics'), {
            'fields': ('description', 'contract', 'supplier', 'externalReference', 'status')
        }),
        (_('Advanced (not editable)'), {
            'classes': ('collapse',),
            'fields': ('lastPricingDate', 'lastCalculatedPrice')
        }),
    )
    allow_add = False


class OptionContract(admin.ModelAdmin):
    list_display = ('id', 'description', 'defaultcustomer', 'defaultSupplier', 'staff', 'defaultcurrency')
    list_display_links = ('id', 'description', 'defaultcustomer', 'defaultSupplier', 'defaultcurrency')
    list_filter = ('defaultcustomer', 'defaultSupplier', 'staff', 'defaultcurrency')
    ordering = ('id', 'defaultcustomer', 'defaultcurrency')
    search_fields = ('id', 'contract', 'defaultcurrency__description')
    fieldsets = (
        (_('Basics'), {
            'fields': ('description', 'defaultcustomer', 'staff', 'defaultSupplier', 'defaultcurrency')
        }),
    )
    inlines = [ContractPostalAddress, ContractPhoneAddress, ContractEmailAddress, InlineQuote, InlineInvoice,
               InlinePurchaseOrder]
    pluginProcessor = PluginProcessor()
    inlines.extend(pluginProcessor.getPluginAdditions("contractInlines"))

    def createPurchaseOrder(self, request, queryset):
        for obj in queryset:
            purchaseorder = obj.createPurchaseOrder()
            self.message_user(request, _("PurchaseOrder created"))
            response = HttpResponseRedirect('/admin/crm/purchaseorder/' + str(purchaseorder.id))
        return response

    createPurchaseOrder.short_description = _("Create Purchaseorder")

    def createQuote(self, request, queryset):
        for obj in queryset:
            quote = obj.createQuote()
            self.message_user(request, _("Quote created"))
            response = HttpResponseRedirect('/admin/crm/quote/' + str(quote.id))
        return response

    createQuote.short_description = _("Create Quote")

    def createInvoice(self, request, queryset):
        for obj in queryset:
            invoice = obj.createInvoice()
            self.message_user(request, _("Invoice created"))
            response = HttpResponseRedirect('/admin/crm/invoice/' + str(invoice.id))
        return response

    createInvoice.short_description = _("Create Invoice")

    def save_model(self, request, obj, form, change):
        if (change == True):
            obj.lastmodifiedby = request.user
        else:
            obj.lastmodifiedby = request.user
            obj.staff = request.user
        obj.save()

    actions = ['createQuote', 'createInvoice', 'createPurchaseOrder']
    pluginProcessor = PluginProcessor()
    actions.extend(pluginProcessor.getPluginAdditions("contractActions"))


class PurchaseOrderInlinePosition(admin.TabularInline):
    model = PurchaseOrderPosition
    extra = 1
    classes = ['collapse']
    fieldsets = (
        ('', {
            'fields': (
            'positionNumber', 'quantity', 'unit', 'product', 'description', 'discount', 'overwriteProductPrice',
            'positionPricePerUnit', 'sentOn', 'supplier')
        }),
    )
    allow_add = True


class InlineBookings(admin.TabularInline):
    model = Booking
    extra = 1
    classes = ['collapse']
    fieldsets = (
        ('Basics', {
            'fields': ('fromAccount', 'toAccount', 'description', 'amount', 'bookingDate', 'staff', 'bookingReference',)
        }),
    )
    allow_add = False


class OptionInvoice(admin.ModelAdmin):
    list_display = (
    'id', 'description', 'contract', 'customer', 'payableuntil', 'status', 'currency', 'staff', 'lastCalculatedPrice',
    'lastPricingDate', 'lastmodification', 'lastmodifiedby')
    list_display_links = ('id', 'contract', 'customer')
    list_filter = ('customer', 'contract', 'staff', 'status', 'currency', 'lastmodification')
    ordering = ('contract', 'customer', 'currency')
    search_fields = ('contract__id', 'customer__name', 'currency__description')
    fieldsets = (
        (_('Basics'), {
            'fields': ('contract', 'description', 'customer', 'currency', 'payableuntil', 'status')
        }),
    )
    save_as = True
    inlines = [SalesContractInlinePosition, SalesContractPostalAddress, SalesContractPhoneAddress,
               SalesContractEmailAddress, InlineBookings]
    pluginProcessor = PluginProcessor()
    inlines.extend(pluginProcessor.getPluginAdditions("invoiceInlines"))

    class PaymentForm(forms.Form):
        paymentAmount = forms.DecimalField()
        _selected_action = forms.CharField(widget=forms.MultipleHiddenInput)
        paymentAccount = forms.ModelChoiceField(Account.objects.filter(accountType="A"))

    def response_add(self, request, new_object):
        obj = self.after_saving_model_and_related_inlines(request, new_object)
        return super(OptionInvoice, self).response_add(request, obj)

    def response_change(self, request, new_object):
        obj = self.after_saving_model_and_related_inlines(request, new_object)
        return super(OptionInvoice, self).response_add(request, obj)

    def after_saving_model_and_related_inlines(self, request, obj):
        try:
            obj.recalculatePrices(date.today())
            self.message_user(request, "Successfully calculated Prices")
        except Product.NoPriceFound as e:
            self.message_user(request, "Unsuccessfull in updating the Prices " + e.__str__())
        return obj

    def save_model(self, request, obj, form, change):
        if (change == True):
            obj.lastmodifiedby = request.user
        else:
            obj.lastmodifiedby = request.user
            obj.staff = request.user
        obj.save()

    def recalculatePrices(self, request, queryset):
        try:
            for obj in queryset:
                obj.recalculatePrices(date.today())
            self.message_user(request, "Successfully recalculated Prices")
        except Product.NoPriceFound as e:
            self.message_user(request, "Unsuccessfull in updating the Prices " + e.__str__())
            return;

    recalculatePrices.short_description = _("Recalculate Prices")

    def createInvoicePDF(self, request, queryset):
        for obj in queryset:
            response = exportPDF(self, request, obj, "invoice", "/admin/crm/invoice/")
            return response

    createInvoicePDF.short_description = _("Create PDF of Invoice")

    def createDeliveryOrderPDF(self, request, queryset):
        for obj in queryset:
            response = exportPDF(self, request, obj, "deliveryorder", "/admin/crm/invoice/")
            return response

    createDeliveryOrderPDF.short_description = _("Create PDF of Delivery Order")

    def registerInvoiceInAccounting(self, request, queryset):
        for obj in queryset:
            obj.registerinvoiceinaccounting(request)
            self.message_user(request, _("Successfully registered Invoice in the Accounting"))

    registerInvoiceInAccounting.short_description = _("Register Invoice in Accounting")

    # def unregisterInvoiceInAccounting(self, request, queryset):
    # for obj in queryset:
    # obj.createPDF(deliveryorder=True)
    # self.message_user(request, _("Successfully unregistered Invoice in the Accounting"))
    # unregisterInvoiceInAccounting.short_description = _("Unregister Invoice in Accounting")

    def registerPaymentInAccounting(self, request, queryset):
        form = None
        if request.POST.get('post'):
            if 'cancel' in request.POST:
                self.message_user(request, _("Canceled registeration of payment in the accounting"))
                return
            elif 'register' in request.POST:
                form = self.PaymentForm(request.POST)
                if form.is_valid():
                    paymentAmount = form.cleaned_data['paymentAmount']
                    paymentAccount = form.cleaned_data['paymentAccount']
                    for obj in queryset:
                        obj.registerpaymentinaccounting(request, paymentAmount, paymentAccount)
                    self.message_user(request, _("Successfully registered Payment in the Accounting"))
                    return HttpResponseRedirect(request.get_full_path())
        else:
            form = self.PaymentForm
            c = {'action_checkbox_name': helpers.ACTION_CHECKBOX_NAME, 'queryset': queryset, 'form': form}
            c.update(csrf(request))
            return render(request, 'crm/admin/registerPayment.html', c)

    registerPaymentInAccounting.short_description = _("Register Payment in Accounting")

    actions = ['recalculatePrices', 'createDeliveryOrderPDF', 'createInvoicePDF', 'registerInvoiceInAccounting',
               'unregisterInvoiceInAccounting', 'registerPaymentInAccounting']
    pluginProcessor = PluginProcessor()
    actions.extend(pluginProcessor.getPluginAdditions("invoiceActions"))


class OptionQuote(admin.ModelAdmin):
    list_display = (
    'id', 'description', 'contract', 'customer', 'currency', 'validuntil', 'status', 'staff', 'lastmodifiedby',
    'lastCalculatedPrice', 'lastPricingDate', 'lastmodification')
    list_display_links = ('id', 'contract', 'customer', 'currency')
    list_filter = ('customer', 'contract', 'currency', 'staff', 'status', 'lastmodification')
    ordering = ('contract', 'customer', 'currency')
    search_fields = ('contract__id', 'customer__name', 'currency__description')

    fieldsets = (
        (_('Basics'), {
            'fields': ('contract', 'description', 'customer', 'currency', 'validuntil', 'staff', 'status')
        }),
    )
    save_as = True
    inlines = [SalesContractInlinePosition, SalesContractPostalAddress, SalesContractPhoneAddress,
               SalesContractEmailAddress]
    pluginProcessor = PluginProcessor()
    inlines.extend(pluginProcessor.getPluginAdditions("quoteInlines"))

    def response_add(self, request, new_object):
        obj = self.after_saving_model_and_related_inlines(request, new_object)
        return super(OptionQuote, self).response_add(request, obj)

    def response_change(self, request, new_object):
        obj = self.after_saving_model_and_related_inlines(request, new_object)
        return super(OptionQuote, self).response_change(request, obj)

    def after_saving_model_and_related_inlines(self, request, obj):
        try:
            obj.recalculatePrices(date.today())
            self.message_user(request, "Successfully calculated Prices")
        except Product.NoPriceFound as e:
            self.message_user(request, "Unsuccessfull in updating the Prices " + e.__str__())
        return obj

    def save_model(self, request, obj, form, change):
        if (change == True):
            obj.lastmodifiedby = request.user
        else:
            obj.lastmodifiedby = request.user
            obj.staff = request.user
        obj.save()

    def recalculatePrices(self, request, queryset):
        try:
            for obj in queryset:
                obj.recalculatePrices(date.today())
                self.message_user(request, _("Successfully recalculated Prices"))
        except Product.NoPriceFound as e:
            self.message_user(request, _("Unsuccessfull in updating the Prices ") + e.__str__())
            return;

    recalculatePrices.short_description = _("Recalculate Prices")

    def createInvoice(self, request, queryset):
        for obj in queryset:
            invoice = obj.createInvoice()
            self.message_user(request, _("Invoice created"))
            response = HttpResponseRedirect('/admin/crm/invoice/' + str(invoice.id))
        return response

    createInvoice.short_description = _("Create Invoice")

    def createQuotePDF(self, request, queryset):
        for obj in queryset:
            response = exportPDF(self, request, obj, "quote", "/admin/crm/quote/")
            return response

    createQuotePDF.short_description = _("Create PDF of Quote")

    def createPurchaseConfirmationPDF(self, request, queryset):
        for obj in queryset:
            response = exportPDF(self, request, obj, "purchaseconfirmation", "/admin/crm/quote/")
            return response

    createPurchaseConfirmationPDF.short_description = _("Create PDF of Purchase Confirmation")

    actions = ['recalculatePrices', 'createInvoice', 'createQuotePDF', 'createPurchaseConfirmationPDF']
    pluginProcessor = PluginProcessor()
    inlines.extend(pluginProcessor.getPluginAdditions("quoteActions"))


class OptionPurchaseOrder(admin.ModelAdmin):
    list_display = (
    'id', 'description', 'contract', 'supplier', 'status', 'currency', 'staff', 'lastmodifiedby', 'lastCalculatedPrice',
    'lastPricingDate', 'lastmodification')
    list_display_links = ('id', 'contract', 'supplier',)
    list_filter = ('supplier', 'contract', 'staff', 'status', 'currency', 'lastmodification')
    ordering = ('contract', 'supplier', 'currency')
    search_fields = ('contract__id', 'supplier__name', 'currency_description')

    fieldsets = (
        (_('Basics'), {
            'fields': ('contract', 'description', 'supplier', 'currency', 'status')
        }),
    )

    def save_model(self, request, obj, form, change):
        if (change == True):
            obj.lastmodifiedby = request.user
        else:
            obj.lastmodifiedby = request.user
            obj.staff = request.user
        obj.save()

    def createPurchseOrderPDF(self, request, queryset):
        for obj in queryset:
            response = exportPDF(self, request, obj, "purchaseorder", "/admin/crm/purchaseorder/")
            return response

    createPurchseOrderPDF.short_description = _("Create PDF of Purchase Order")

    actions = ['createPurchseOrderPDF']
    pluginProcessor = PluginProcessor()
    actions.extend(pluginProcessor.getPluginAdditions("purchaseOrderActions"))

    save_as = True
    inlines = [PurchaseOrderInlinePosition, PurchaseOrderPostalAddress, PurchaseOrderPhoneAddress,
               PurchaseOrderEmailAddress]
    pluginProcessor = PluginProcessor()
    inlines.extend(pluginProcessor.getPluginAdditions("purchaseOrderInlines"))


class ProductPrice(admin.TabularInline):
    model = Price
    extra = 1
    classes = ['collapse']
    fieldsets = (
        ('', {
            'fields': ('price', 'validfrom', 'validuntil', 'unit', 'customerGroup', 'currency')
        }),
    )
    allow_add = True


class ProductUnitTransform(admin.TabularInline):
    model = UnitTransform
    extra = 1
    classes = ['collapse']
    fieldsets = (
        ('', {
            'fields': ('fromUnit', 'toUnit', 'factor',)
        }),
    )
    allow_add = True


class OptionProduct(admin.ModelAdmin):
    list_display = ('productNumber', 'title', 'defaultunit', 'tax', 'accoutingProductCategorie')
    list_display_links = ('productNumber',)
    fieldsets = (
        (_('Basics'), {
            'fields': ('productNumber', 'title', 'description', 'defaultunit', 'tax', 'accoutingProductCategorie')
        }),)
    inlines = [ProductPrice, ProductUnitTransform]


class ContactPostalAddress(admin.StackedInline):
    model = PostalAddressForContact
    extra = 1
    classes = ['collapse']
    fieldsets = (
        ('Basics', {
            'fields': (
            'prefix', 'prename', 'name', 'addressline1', 'addressline2', 'addressline3', 'addressline4', 'zipcode',
            'town', 'state', 'country', 'purpose')
        }),
    )
    allow_add = True


class ContactPhoneAddress(admin.TabularInline):
    model = PhoneAddressForContact
    extra = 1
    classes = ['collapse']
    fieldsets = (
        ('Basics', {
            'fields': ('phone', 'purpose',)
        }),
    )
    allow_add = True


class ContactEmailAddress(admin.TabularInline):
    model = EmailAddressForContact
    extra = 1
    classes = ['collapse']
    fieldsets = (
        ('Basics', {
            'fields': ('email', 'purpose',)
        }),
    )
    allow_add = True


class OptionCustomer(admin.ModelAdmin):
    list_display = ('id', 'name', 'defaultCustomerBillingCycle',)
    fieldsets = (('', {'fields': ('name', 'defaultCustomerBillingCycle', 'ismemberof',)}),)
    allow_add = True
    ordering = ('id', 'name')
    search_fields = ('id', 'name')
    inlines = [ContactPostalAddress, ContactPhoneAddress, ContactEmailAddress]
    pluginProcessor = PluginProcessor()
    inlines.extend(pluginProcessor.getPluginAdditions("customerInline"))

    def createContract(self, request, queryset):
        for obj in queryset:
            contract = obj.createContract(request)
            response = HttpResponseRedirect('/admin/crm/contract/' + str(contract.id))
        return response

    createContract.short_description = _("Create Contract")

    def createQuote(self, request, queryset):
        for obj in queryset:
            quote = obj.createQuote()
            response = HttpResponseRedirect('/admin/crm/quote/' + str(quote.id))
        return response

    createQuote.short_description = _("Create Quote")

    def createInvoice(self, request, queryset):
        for obj in queryset:
            invoice = obj.createInvoice()
            response = HttpResponseRedirect('/admin/crm/invoice/' + str(invoice.id))
        return response

    createInvoice.short_description = _("Create Invoice")

    def save_model(self, request, obj, form, change):
        if (change == True):
            obj.lastmodifiedby = request.user
        else:
            obj.lastmodifiedby = request.user
            obj.staff = request.user
        obj.save()

    actions = ['createContract', 'createInvoice', 'createQuote']
    pluginProcessor = PluginProcessor()
    inlines.extend(pluginProcessor.getPluginAdditions("customerActions"))


class OptionCustomerGroup(admin.ModelAdmin):
    list_display = ('id', 'name')
    fieldsets = (('', {'fields': ('name',)}),)
    allow_add = True


class OptionSupplier(admin.ModelAdmin):
    list_display = ('id', 'name', 'offersShipmentToCustomers')
    fieldsets = (('', {'fields': ('name', 'offersShipmentToCustomers')}),)
    inlines = [ContactPostalAddress, ContactPhoneAddress, ContactEmailAddress]
    allow_add = True

    def save_model(self, request, obj, form, change):
        if (change == True):
            obj.lastmodifiedby = request.user
        else:
            obj.lastmodifiedby = request.user
            obj.staff = request.user
        obj.save()


class OptionUnit(admin.ModelAdmin):
    list_display = ('id', 'description', 'shortName', 'isAFractionOf', 'fractionFactorToNextHigherUnit')
    fieldsets = (('', {'fields': ('description', 'shortName', 'isAFractionOf', 'fractionFactorToNextHigherUnit')}),)
    allow_add = True


class OptionCurrency(admin.ModelAdmin):
    list_display = ('id', 'description', 'shortName', 'rounding')
    fieldsets = (('', {'fields': ('description', 'shortName', 'rounding')}),)
    allow_add = True


class OptionTax(admin.ModelAdmin):
    list_display = ('id', 'taxrate', 'name', 'accountActiva', 'accountPassiva')
    fieldsets = (('', {'fields': ('taxrate', 'name', 'accountActiva', 'accountPassiva')}),)
    allow_add = True


class OptionCustomerBillingCycle(admin.ModelAdmin):
    list_display = ('id', 'timeToPaymentDate', 'name')
    fieldsets = (('', {'fields': ('timeToPaymentDate', 'name',)}),)
    allow_add = True


admin.site.register(Customer, OptionCustomer)
admin.site.register(CustomerGroup, OptionCustomerGroup)
admin.site.register(Supplier, OptionSupplier)
admin.site.register(Quote, OptionQuote)
admin.site.register(Invoice, OptionInvoice)
admin.site.register(Unit, OptionUnit)
admin.site.register(Currency, OptionCurrency)
admin.site.register(Tax, OptionTax)
admin.site.register(CustomerBillingCycle, OptionCustomerBillingCycle)
admin.site.register(Contract, OptionContract)
admin.site.register(PurchaseOrder, OptionPurchaseOrder)
admin.site.register(Product, OptionProduct)
